@@ -10,29 +10,30 @@
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 2, as
- * published by the Free Software Foundation.
+ * it under the terms of the GNU Lesser General Public License version 2.1,
+ * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: dtmf.c,v 1.21 2007/06/26 15:54:12 steveu Exp $
+ * $Id: dtmf.c,v 1.43 2008/07/02 14:48:25 steveu Exp $
  */
  
 /*! \file dtmf.h */
 
-#ifdef HAVE_CONFIG_H
+#if defined(HAVE_CONFIG_H)
 #include "config.h"
 #endif
 
 #include <inttypes.h>
 #include <stdlib.h>
+#include "floating_fudge.h"
 #if defined(HAVE_TGMATH_H)
 #include <tgmath.h>
 #endif
@@ -46,38 +47,35 @@
 
 #include "spandsp/telephony.h"
 #include "spandsp/queue.h"
+#include "spandsp/complex.h"
+#include "spandsp/dds.h"
 #include "spandsp/tone_detect.h"
 #include "spandsp/tone_generate.h"
 #include "spandsp/super_tone_rx.h"
 #include "spandsp/dtmf.h"
 
-#if !defined(M_PI)
-/* C99 systems may not define M_PI */
-#define M_PI 3.14159265358979323846264338327
-#endif
-
-//#define USE_3DNOW
-
 #define DEFAULT_DTMF_TX_LEVEL       -10
 #define DEFAULT_DTMF_TX_ON_TIME     50
 #define DEFAULT_DTMF_TX_OFF_TIME    55
 
-#if defined(USE_FIXED_POINT_EXPERIMENTAL)
-#define DTMF_THRESHOLD              1.0e5f
-#define DTMF_NORMAL_TWIST           6.3f    /* 8dB */
-#define DTMF_REVERSE_TWIST          2.5f    /* 4dB */
-#define DTMF_RELATIVE_PEAK_ROW      6.3f    /* 8dB */
-#define DTMF_RELATIVE_PEAK_COL      6.3f    /* 8dB */
-#define DTMF_TO_TOTAL_ENERGY        42.0f
-#define DTMF_POWER_OFFSET           (90.30f - 48.32f)
+#if defined(SPANDSP_USE_FIXED_POINT)
+#define DTMF_THRESHOLD              10438           /* -42dBm0 */
+#define DTMF_NORMAL_TWIST           6.309f          /* 8dB */
+#define DTMF_REVERSE_TWIST          2.512f          /* 4dB */
+#define DTMF_RELATIVE_PEAK_ROW      6.309f          /* 8dB */
+#define DTMF_RELATIVE_PEAK_COL      6.309f          /* 8dB */
+#define DTMF_TO_TOTAL_ENERGY        83.868f         /* -0.85dB */
+#define DTMF_POWER_OFFSET           68.251f         /* 10*log(256.0*256.0*DTMF_SAMPLES_PER_BLOCK) */
+#define DTMF_SAMPLES_PER_BLOCK      102
 #else
-#define DTMF_THRESHOLD              8.0e7f
-#define DTMF_NORMAL_TWIST           6.3f    /* 8dB */
-#define DTMF_REVERSE_TWIST          2.5f    /* 4dB */
-#define DTMF_RELATIVE_PEAK_ROW      6.3f    /* 8dB */
-#define DTMF_RELATIVE_PEAK_COL      6.3f    /* 8dB */
-#define DTMF_TO_TOTAL_ENERGY        42.0f
-#define DTMF_POWER_OFFSET           90.30f
+#define DTMF_THRESHOLD              171032462.0f    /* -42dBm0 [((DTMF_SAMPLES_PER_BLOCK*32768.0/1.4142)*10^((-42 - DBM0_MAX_SINE_POWER)/20.0))^2 => 171032462.0] */
+#define DTMF_NORMAL_TWIST           6.309f          /* 8dB [10^(8/10) => 6.309] */
+#define DTMF_REVERSE_TWIST          2.512f          /* 4dB */
+#define DTMF_RELATIVE_PEAK_ROW      6.309f          /* 8dB */
+#define DTMF_RELATIVE_PEAK_COL      6.309f          /* 8dB */
+#define DTMF_TO_TOTAL_ENERGY        83.868f         /* -0.85dB [DTMF_SAMPLES_PER_BLOCK*10^(-0.85/10.0)] */
+#define DTMF_POWER_OFFSET           110.395f        /* 10*log(32768.0*32768.0*DTMF_SAMPLES_PER_BLOCK) */
+#define DTMF_SAMPLES_PER_BLOCK      102
 #endif
 
 static const float dtmf_row[] =
@@ -97,102 +95,20 @@ static goertzel_descriptor_t dtmf_detect_col[4];
 static int dtmf_tx_inited = FALSE;
 static tone_gen_descriptor_t dtmf_digit_tones[16];
 
-#if defined(USE_3DNOW)
-static __inline__ void _dtmf_goertzel_update(goertzel_state_t *s,
-                                             float x[],
-                                             int samples)
-{
-    int n;
-    float v;
-    int i;
-    float vv[16];
-
-    vv[4] = s[0].v2;
-    vv[5] = s[1].v2;
-    vv[6] = s[2].v2;
-    vv[7] = s[3].v2;
-    vv[8] = s[0].v3;
-    vv[9] = s[1].v3;
-    vv[10] = s[2].v3;
-    vv[11] = s[3].v3;
-    vv[12] = s[0].fac;
-    vv[13] = s[1].fac;
-    vv[14] = s[2].fac;
-    vv[15] = s[3].fac;
-
-    //v1 = s->v2;
-    //s->v2 = s->v3;
-    //s->v3 = s->fac*s->v2 - v1 + x[0];
-
-    __asm__ __volatile__ (
-             " femms;\n"
-
-             " movq        16(%%edx),%%mm2;\n"
-             " movq        24(%%edx),%%mm3;\n"
-             " movq        32(%%edx),%%mm4;\n"
-             " movq        40(%%edx),%%mm5;\n"
-             " movq        48(%%edx),%%mm6;\n"
-             " movq        56(%%edx),%%mm7;\n"
-
-             " jmp         1f;\n"
-             " .align 32;\n"
-
-             " 1: ;\n"
-             " prefetch    (%%eax);\n"
-             " movq        %%mm3,%%mm1;\n"
-             " movq        %%mm2,%%mm0;\n"
-             " movq        %%mm5,%%mm3;\n"
-             " movq        %%mm4,%%mm2;\n"
-
-             " pfmul       %%mm7,%%mm5;\n"
-             " pfmul       %%mm6,%%mm4;\n"
-             " pfsub       %%mm1,%%mm5;\n"
-             " pfsub       %%mm0,%%mm4;\n"
-
-             " movq        (%%eax),%%mm0;\n"
-             " movq        %%mm0,%%mm1;\n"
-             " punpckldq   %%mm0,%%mm1;\n"
-             " add         $4,%%eax;\n"
-             " pfadd       %%mm1,%%mm5;\n"
-             " pfadd       %%mm1,%%mm4;\n"
-
-             " dec         %%ecx;\n"
-
-             " jnz         1b;\n"
-
-             " movq        %%mm2,16(%%edx);\n"
-             " movq        %%mm3,24(%%edx);\n"
-             " movq        %%mm4,32(%%edx);\n"
-             " movq        %%mm5,40(%%edx);\n"
-
-             " femms;\n"
-             :
-             : "c" (samples), "a" (x), "d" (vv)
-             : "memory", "eax", "ecx");
-
-    s[0].v2 = vv[4];
-    s[1].v2 = vv[5];
-    s[2].v2 = vv[6];
-    s[3].v2 = vv[7];
-    s[0].v3 = vv[8];
-    s[1].v3 = vv[9];
-    s[2].v3 = vv[10];
-    s[3].v3 = vv[11];
-}
-/*- End of function --------------------------------------------------------*/
-#endif
-
 int dtmf_rx(dtmf_rx_state_t *s, const int16_t amp[], int samples)
 {
+#if defined(SPANDSP_USE_FIXED_POINT)
+    int32_t row_energy[4];
+    int32_t col_energy[4];
+    int16_t xamp;
+    float famp;
+#else
     float row_energy[4];
     float col_energy[4];
-#if defined(USE_FIXED_POINT_EXPERIMENTAL)
-    int famp;
-    int v1;
-#else
+    float xamp;
     float famp;
-    float v1;
 #endif
+    float v1;
     int i;
     int j;
     int sample;
@@ -205,110 +121,49 @@ int dtmf_rx(dtmf_rx_state_t *s, const int16_t amp[], int samples)
     for (sample = 0;  sample < samples;  sample = limit)
     {
         /* The block length is optimised to meet the DTMF specs. */
-        if ((samples - sample) >= (102 - s->current_sample))
-            limit = sample + (102 - s->current_sample);
+        if ((samples - sample) >= (DTMF_SAMPLES_PER_BLOCK - s->current_sample))
+            limit = sample + (DTMF_SAMPLES_PER_BLOCK - s->current_sample);
         else
             limit = samples;
-#if defined(USE_3DNOW)
-        _dtmf_goertzel_update(s->row_out, amp + sample, limit - sample);
-        _dtmf_goertzel_update(s->col_out, amp + sample, limit - sample);
-#else
         /* The following unrolled loop takes only 35% (rough estimate) of the 
            time of a rolled loop on the machine on which it was developed */
         for (j = sample;  j < limit;  j++)
         {
-            famp = amp[j];
+            xamp = amp[j];
             if (s->filter_dialtone)
             {
+                famp = xamp;
                 /* Sharp notches applied at 350Hz and 440Hz - the two common dialtone frequencies.
                    These are rather high Q, to achieve the required narrowness, without using lots of
                    sections. */
-                v1 = 0.98356f*famp + 1.8954426f*s->z350_1 - 0.9691396f*s->z350_2;
-                famp = v1 - 1.9251480f*s->z350_1 + s->z350_2;
-                s->z350_2 = s->z350_1;
-                s->z350_1 = v1;
+                v1 = 0.98356f*famp + 1.8954426f*s->z350[0] - 0.9691396f*s->z350[1];
+                famp = v1 - 1.9251480f*s->z350[0] + s->z350[1];
+                s->z350[1] = s->z350[0];
+                s->z350[0] = v1;
 
-                v1 = 0.98456f*famp + 1.8529543f*s->z440_1 - 0.9691396f*s->z440_2;
-                famp = v1 - 1.8819938f*s->z440_1 + s->z440_2;
-                s->z440_2 = s->z440_1;
-                s->z440_1 = v1;
+                v1 = 0.98456f*famp + 1.8529543f*s->z440[0] - 0.9691396f*s->z440[1];
+                famp = v1 - 1.8819938f*s->z440[0] + s->z440[1];
+                s->z440[1] = s->z440[0];
+                s->z440[0] = v1;
+                xamp = famp;
             }
-#if defined(USE_FIXED_POINT_EXPERIMENTAL)
-            famp >>= 8;
-            s->energy += famp*famp;
-            /* With GCC 2.95, the following unrolled code seems to take about 35%
-               (rough estimate) as long as a neat little 0-3 loop */
-            v1 = s->row_out[0].v2;
-            s->row_out[0].v2 = s->row_out[0].v3;
-            s->row_out[0].v3 = ((s->row_out[0].fac*s->row_out[0].v2) >> 12) - v1 + famp;
-
-            v1 = s->col_out[0].v2;
-            s->col_out[0].v2 = s->col_out[0].v3;
-            s->col_out[0].v3 = ((s->col_out[0].fac*s->col_out[0].v2) >> 12) - v1 + famp;
-
-            v1 = s->row_out[1].v2;
-            s->row_out[1].v2 = s->row_out[1].v3;
-            s->row_out[1].v3 = ((s->row_out[1].fac*s->row_out[1].v2) >> 12) - v1 + famp;
-
-            v1 = s->col_out[1].v2;
-            s->col_out[1].v2 = s->col_out[1].v3;
-            s->col_out[1].v3 = ((s->col_out[1].fac*s->col_out[1].v2) >> 12) - v1 + famp;
-
-            v1 = s->row_out[2].v2;
-            s->row_out[2].v2 = s->row_out[2].v3;
-            s->row_out[2].v3 = ((s->row_out[2].fac*s->row_out[2].v2) >> 12) - v1 + famp;
-
-            v1 = s->col_out[2].v2;
-            s->col_out[2].v2 = s->col_out[2].v3;
-            s->col_out[2].v3 = ((s->col_out[2].fac*s->col_out[2].v2) >> 12) - v1 + famp;
-
-            v1 = s->row_out[3].v2;
-            s->row_out[3].v2 = s->row_out[3].v3;
-            s->row_out[3].v3 = ((s->row_out[3].fac*s->row_out[3].v2) >> 12) - v1 + famp;
-
-            v1 = s->col_out[3].v2;
-            s->col_out[3].v2 = s->col_out[3].v3;
-            s->col_out[3].v3 = ((s->col_out[3].fac*s->col_out[3].v2) >> 12) - v1 + famp;
+            xamp = goertzel_preadjust_amp(xamp);
+#if defined(SPANDSP_USE_FIXED_POINT)
+            s->energy += ((int32_t) xamp*xamp);
 #else
-            s->energy += famp*famp;
-            /* With GCC 2.95, the following unrolled code seems to take about 35%
-               (rough estimate) as long as a neat little 0-3 loop */
-            v1 = s->row_out[0].v2;
-            s->row_out[0].v2 = s->row_out[0].v3;
-            s->row_out[0].v3 = s->row_out[0].fac*s->row_out[0].v2 - v1 + famp;
-
-            v1 = s->col_out[0].v2;
-            s->col_out[0].v2 = s->col_out[0].v3;
-            s->col_out[0].v3 = s->col_out[0].fac*s->col_out[0].v2 - v1 + famp;
-
-            v1 = s->row_out[1].v2;
-            s->row_out[1].v2 = s->row_out[1].v3;
-            s->row_out[1].v3 = s->row_out[1].fac*s->row_out[1].v2 - v1 + famp;
-
-            v1 = s->col_out[1].v2;
-            s->col_out[1].v2 = s->col_out[1].v3;
-            s->col_out[1].v3 = s->col_out[1].fac*s->col_out[1].v2 - v1 + famp;
-
-            v1 = s->row_out[2].v2;
-            s->row_out[2].v2 = s->row_out[2].v3;
-            s->row_out[2].v3 = s->row_out[2].fac*s->row_out[2].v2 - v1 + famp;
-
-            v1 = s->col_out[2].v2;
-            s->col_out[2].v2 = s->col_out[2].v3;
-            s->col_out[2].v3 = s->col_out[2].fac*s->col_out[2].v2 - v1 + famp;
-
-            v1 = s->row_out[3].v2;
-            s->row_out[3].v2 = s->row_out[3].v3;
-            s->row_out[3].v3 = s->row_out[3].fac*s->row_out[3].v2 - v1 + famp;
-
-            v1 = s->col_out[3].v2;
-            s->col_out[3].v2 = s->col_out[3].v3;
-            s->col_out[3].v3 = s->col_out[3].fac*s->col_out[3].v2 - v1 + famp;
+            s->energy += xamp*xamp;
 #endif
+            goertzel_samplex(&s->row_out[0], xamp);
+            goertzel_samplex(&s->col_out[0], xamp);
+            goertzel_samplex(&s->row_out[1], xamp);
+            goertzel_samplex(&s->col_out[1], xamp);
+            goertzel_samplex(&s->row_out[2], xamp);
+            goertzel_samplex(&s->col_out[2], xamp);
+            goertzel_samplex(&s->row_out[3], xamp);
+            goertzel_samplex(&s->col_out[3], xamp);
         }
-#endif
         s->current_sample += (limit - sample);
-        if (s->current_sample < 102)
+        if (s->current_sample < DTMF_SAMPLES_PER_BLOCK)
             continue;
 
         /* We are at the end of a DTMF detection block */
@@ -317,7 +172,6 @@ int dtmf_rx(dtmf_rx_state_t *s, const int16_t amp[], int samples)
         best_row = 0;
         col_energy[0] = goertzel_result(&s->col_out[0]);
         best_col = 0;
-
         for (i = 1;  i < 4;  i++)
         {
             row_energy[i] = goertzel_result(&s->row_out[i]);
@@ -329,9 +183,9 @@ int dtmf_rx(dtmf_rx_state_t *s, const int16_t amp[], int samples)
         }
         hit = 0;
         /* Basic signal level test and the twist test */
-        if (row_energy[best_row] >= DTMF_THRESHOLD
+        if (row_energy[best_row] >= s->threshold
             &&
-            col_energy[best_col] >= DTMF_THRESHOLD
+            col_energy[best_col] >= s->threshold
             &&
             col_energy[best_col] < row_energy[best_row]*s->reverse_twist
             &&
@@ -393,8 +247,8 @@ int dtmf_rx(dtmf_rx_state_t *s, const int16_t amp[], int samples)
                     /* Avoid reporting multiple no digit conditions on flaky hits */
                     if (s->in_digit  ||  hit)
                     {
-                        i = (s->in_digit  &&  !hit)  ?  -99  :  rintf(log10f(s->energy)*10.0f - 20.08f - DTMF_POWER_OFFSET + DBM0_MAX_POWER);
-                        s->realtime_callback(s->realtime_callback_data, hit, i);
+                        i = (s->in_digit  &&  !hit)  ?  -99  :  lrintf(log10f(s->energy)*10.0f - DTMF_POWER_OFFSET + DBM0_MAX_POWER);
+                        s->realtime_callback(s->realtime_callback_data, hit, i, 0);
                     }
                 }
                 else
@@ -405,9 +259,9 @@ int dtmf_rx(dtmf_rx_state_t *s, const int16_t amp[], int samples)
                         {
                             s->digits[s->current_digits++] = (char) hit;
                             s->digits[s->current_digits] = '\0';
-                            if (s->callback)
+                            if (s->digits_callback)
                             {
-                                s->callback(s->callback_data, s->digits, s->current_digits);
+                                s->digits_callback(s->digits_callback_data, s->digits, s->current_digits);
                                 s->current_digits = 0;
                             }
                         }
@@ -421,18 +275,16 @@ int dtmf_rx(dtmf_rx_state_t *s, const int16_t amp[], int samples)
             }
         }
         s->last_hit = hit;
-        /* Reinitialise the detector for the next block */
-        for (i = 0;  i < 4;  i++)
-        {
-            goertzel_reset(&s->row_out[i]);
-            goertzel_reset(&s->col_out[i]);
-        }
+#if defined(SPANDSP_USE_FIXED_POINT)
+        s->energy = 0;
+#else
         s->energy = 0.0f;
+#endif
         s->current_sample = 0;
     }
-    if (s->current_digits  &&  s->callback)
+    if (s->current_digits  &&  s->digits_callback)
     {
-        s->callback(s->callback_data, s->digits, s->current_digits);
+        s->digits_callback(s->digits_callback_data, s->digits, s->current_digits);
         s->digits[0] = '\0';
         s->current_digits = 0;
     }
@@ -477,37 +329,51 @@ void dtmf_rx_set_realtime_callback(dtmf_rx_state_t *s,
 void dtmf_rx_parms(dtmf_rx_state_t *s,
                    int filter_dialtone,
                    int twist,
-                   int reverse_twist)
+                   int reverse_twist,
+                   int threshold)
 {
+    float x;
+
     if (filter_dialtone >= 0)
     {
-        s->z350_1 = 0.0f;
-        s->z350_2 = 0.0f;
-        s->z440_1 = 0.0f;
-        s->z440_2 = 0.0f;
+        s->z350[0] = 0.0f;
+        s->z350[1] = 0.0f;
+        s->z440[0] = 0.0f;
+        s->z440[1] = 0.0f;
         s->filter_dialtone = filter_dialtone;
     }
     if (twist >= 0)
         s->normal_twist = powf(10.0f, twist/10.0f);
     if (reverse_twist >= 0)
         s->reverse_twist = powf(10.0f, reverse_twist/10.0f);
+    if (threshold > -99)
+    {
+        x = (DTMF_SAMPLES_PER_BLOCK*32768.0f/1.4142f)*powf(10.0f, (threshold - DBM0_MAX_SINE_POWER)/20.0f);
+        s->threshold = x*x;
+    }
 }
 /*- End of function --------------------------------------------------------*/
 
 dtmf_rx_state_t *dtmf_rx_init(dtmf_rx_state_t *s,
-                              dtmf_rx_callback_t callback,
+                              digits_rx_callback_t callback,
                               void *user_data)
 {
     int i;
     static int initialised = FALSE;
 
-    s->callback = callback;
-    s->callback_data = user_data;
+    if (s == NULL)
+    {
+        if ((s = (dtmf_rx_state_t *) malloc(sizeof (*s))) == NULL)
+            return  NULL;
+    }
+    s->digits_callback = callback;
+    s->digits_callback_data = user_data;
     s->realtime_callback = NULL;
     s->realtime_callback_data = NULL;
     s->filter_dialtone = FALSE;
     s->normal_twist = DTMF_NORMAL_TWIST;
     s->reverse_twist = DTMF_REVERSE_TWIST;
+    s->threshold = DTMF_THRESHOLD;
 
     s->in_digit = 0;
     s->last_hit = 0;
@@ -516,8 +382,8 @@ dtmf_rx_state_t *dtmf_rx_init(dtmf_rx_state_t *s,
     {
         for (i = 0;  i < 4;  i++)
         {
-            make_goertzel_descriptor(&dtmf_detect_row[i], dtmf_row[i], 102);
-            make_goertzel_descriptor(&dtmf_detect_col[i], dtmf_col[i], 102);
+            make_goertzel_descriptor(&dtmf_detect_row[i], dtmf_row[i], DTMF_SAMPLES_PER_BLOCK);
+            make_goertzel_descriptor(&dtmf_detect_col[i], dtmf_col[i], DTMF_SAMPLES_PER_BLOCK);
         }
         initialised = TRUE;
     }
@@ -526,7 +392,11 @@ dtmf_rx_state_t *dtmf_rx_init(dtmf_rx_state_t *s,
         goertzel_init(&s->row_out[i], &dtmf_detect_row[i]);
         goertzel_init(&s->col_out[i], &dtmf_detect_col[i]);
     }
+#if defined(SPANDSP_USE_FIXED_POINT)
+    s->energy = 0;
+#else
     s->energy = 0.0f;
+#endif
     s->current_sample = 0;
     s->lost_digits = 0;
     s->current_digits = 0;
@@ -535,6 +405,13 @@ dtmf_rx_state_t *dtmf_rx_init(dtmf_rx_state_t *s,
 }
 /*- End of function --------------------------------------------------------*/
 
+int dtmf_rx_free(dtmf_rx_state_t *s)
+{
+    free(s);
+    return 0;
+}
+/*- End of function --------------------------------------------------------*/
+
 static void dtmf_tx_initialise(void)
 {
     int row;
@@ -574,46 +451,80 @@ int dtmf_tx(dtmf_tx_state_t *s, int16_t amp[], int max_samples)
         /* Deal with the fragment left over from last time */
         len = tone_gen(&(s->tones), amp, max_samples);
     }
-    while (len < max_samples  &&  (digit = queue_read_byte(&s->queue)) >= 0)
+    while (len < max_samples  &&  (digit = queue_read_byte(&s->queue.queue)) >= 0)
     {
         /* Step to the next digit */
+        if (digit == 0)
+            continue;
         if ((cp = strchr(dtmf_positions, digit)) == NULL)
             continue;
-        tone_gen_init(&(s->tones), &(s->tone_descriptors[cp - dtmf_positions]));
+        tone_gen_init(&(s->tones), &dtmf_digit_tones[cp - dtmf_positions]);
+        s->tones.tone[0].gain = s->low_level;
+        s->tones.tone[1].gain = s->high_level;
+        s->tones.duration[0] = s->on_time;
+        s->tones.duration[1] = s->off_time;
         len += tone_gen(&(s->tones), amp + len, max_samples - len);
     }
     return len;
 }
 /*- End of function --------------------------------------------------------*/
 
-size_t dtmf_tx_put(dtmf_tx_state_t *s, const char *digits)
+size_t dtmf_tx_put(dtmf_tx_state_t *s, const char *digits, int len)
 {
-    size_t len;
     size_t space;
 
     /* This returns the number of characters that would not fit in the buffer.
        The buffer will only be loaded if the whole string of digits will fit,
        in which case zero is returned. */
-    if ((len = strlen(digits)) == 0)
-        return 0;
-    if ((space = queue_free_space(&s->queue)) < len)
+    if (len < 0)
+    {
+        if ((len = strlen(digits)) == 0)
+            return 0;
+    }
+    if ((space = queue_free_space(&s->queue.queue)) < len)
         return len - space;
-    if (queue_write(&s->queue, (const uint8_t *) digits, len) >= 0)
+    if (queue_write(&s->queue.queue, (const uint8_t *) digits, len) >= 0)
         return 0;
     return -1;
 }
 /*- End of function --------------------------------------------------------*/
 
+void dtmf_tx_set_level(dtmf_tx_state_t *s, int level, int twist)
+{
+    s->low_level = dds_scaling_dbm0f((float) level);
+    s->high_level = dds_scaling_dbm0f((float) (level + twist));
+}
+/*- End of function --------------------------------------------------------*/
+
+void dtmf_tx_set_timing(dtmf_tx_state_t *s, int on_time, int off_time)
+{
+    s->on_time = ((on_time >= 0)  ?  on_time  :  DEFAULT_DTMF_TX_ON_TIME)*SAMPLE_RATE/1000;
+    s->off_time = ((off_time >= 0)  ?  off_time  :  DEFAULT_DTMF_TX_OFF_TIME)*SAMPLE_RATE/1000;
+}
+/*- End of function --------------------------------------------------------*/
+
 dtmf_tx_state_t *dtmf_tx_init(dtmf_tx_state_t *s)
 {
+    if (s == NULL)
+    {
+        if ((s = (dtmf_tx_state_t *) malloc(sizeof (*s))) == NULL)
+            return  NULL;
+    }
     if (!dtmf_tx_inited)
         dtmf_tx_initialise();
-    s->tone_descriptors = dtmf_digit_tones;
     tone_gen_init(&(s->tones), &dtmf_digit_tones[0]);
-    s->current_sample = 0;
-    queue_init(&s->queue, MAX_DTMF_DIGITS, QUEUE_READ_ATOMIC | QUEUE_WRITE_ATOMIC);
+    dtmf_tx_set_level(s, DEFAULT_DTMF_TX_LEVEL, 0);
+    dtmf_tx_set_timing(s, -1, -1);
+    queue_init(&s->queue.queue, MAX_DTMF_DIGITS, QUEUE_READ_ATOMIC | QUEUE_WRITE_ATOMIC);
     s->tones.current_section = -1;
     return s;
 }
 /*- End of function --------------------------------------------------------*/
+
+int dtmf_tx_free(dtmf_tx_state_t *s)
+{
+    free(s);
+    return 0;
+}
+/*- End of function --------------------------------------------------------*/
 /*- End of file ------------------------------------------------------------*/
