@@ -15,19 +15,19 @@
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 2, as
- * published by the Free Software Foundation.
+ * it under the terms of the GNU Lesser General Public License version 2.1,
+ * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: echo.c,v 1.19 2006/10/24 13:45:25 steveu Exp $
+ * $Id: echo.c,v 1.24 2008/05/13 13:17:22 steveu Exp $
  */
 
 /*! \file */
@@ -78,7 +78,7 @@
    for the 32 bit values. We loose the fractions, but they should soon settle down in a
    reasonable way. */
 
-#ifdef HAVE_CONFIG_H
+#if defined(HAVE_CONFIG_H)
 #include <config.h>
 #endif
 
@@ -232,8 +232,7 @@ echo_can_state_t *echo_can_create(int len, int adaption_mode)
     int i;
     int j;
 
-    ec = (echo_can_state_t *) malloc(sizeof(*ec));
-    if (ec == NULL)
+    if ((ec = (echo_can_state_t *) malloc(sizeof(*ec))) == NULL)
         return  NULL;
     memset(ec, 0, sizeof(*ec));
     ec->taps = len;
@@ -361,7 +360,7 @@ sample_no++;
 
     /* And the answer is..... */
     clean_rx = rx - echo_value;
-printf("echo is %d\n", echo_value);
+printf("echo is %" PRId32 "\n", echo_value);
     /* That was the easy part. Now we need to adapt! */
     if (ec->nonupdate_dwell > 0)
         ec->nonupdate_dwell--;
