@@ -10,33 +10,39 @@
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 2, as
- * published by the Free Software Foundation.
+ * it under the terms of the GNU Lesser General Public License version 2.1,
+ * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  * This code is based on the U.S. Department of Defense reference
  * implementation of the LPC-10 2400 bps Voice Coder. They do not
  * exert copyright claims on their code, and it may be freely used.
  *
- * $Id: lpc10_decode.c,v 1.11 2006/10/24 13:45:25 steveu Exp $
+ * $Id: lpc10_decode.c,v 1.22 2008/07/02 14:48:25 steveu Exp $
  */
 
-#ifdef HAVE_CONFIG_H
+#if defined(HAVE_CONFIG_H)
 #include <config.h>
 #endif
 
 #include <stdlib.h>
 #include <stdio.h>
 #include <inttypes.h>
+#include "floating_fudge.h"
+#if defined(HAVE_TGMATH_H)
 #include <tgmath.h>
+#endif
+#if defined(HAVE_MATH_H)
+#include <math.h>
+#endif
 #include <memory.h>
 
 #include "spandsp/telephony.h"
@@ -45,8 +51,12 @@
 
 #define LPC10_ORDER     10
 
+#if !defined(min)
 #define min(a,b) ((a) <= (b) ? (a) : (b))
+#endif
+#if !defined(max)
 #define max(a,b) ((a) >= (b) ? (a) : (b))
+#endif
 
 /* Pseudo random number generator based on Knuth, Vol 2, p. 27. */
 /* lpc10_random - int32_t variable, uniformly distributed over -32768 to 32767 */
@@ -148,7 +158,7 @@ static void bsynz(lpc10_decode_state_t *s,
     }
     else
     {
-        sscale = sqrt((float) ip)/6.928f;
+        sscale = sqrtf((float) ip)/6.928f;
         for (i = 0;  i < ip;  i++)
         {
             s->exc[LPC10_ORDER + i] = 0.0f;
@@ -200,7 +210,7 @@ static void bsynz(lpc10_decode_state_t *s,
     }
     /* Apply gain to match RMS */
     ssq = rms*rms*ip;
-    gain = sqrt(ssq/xssq);
+    gain = sqrtf(ssq/xssq);
     for (i = 0;  i < ip;  i++)
         sout[i] = gain*s->exc2[LPC10_ORDER + i];
 }
@@ -404,7 +414,7 @@ static int pitsyn(lpc10_decode_state_t *s,
         /* Column 101 is identical to 100.  Any good properties we can prove
            for 100 will also hold for 101.  Similarly for 010 and 011. */
 
-        /* SYNTHS calls this subroutine with PITCH restricted to the range 20 to
+        /* synths() calls this subroutine with PITCH restricted to the range 20 to
            156.  IPITO is similarly restricted to this range, after the first
            call.  IP below is also restricted to this range, given the
            definitions of IPITO, SLOPE, UVPIT, and that I is in the range ISTART
@@ -415,9 +425,9 @@ static int pitsyn(lpc10_decode_state_t *s,
             for (i = istart;  i <= lsamp;  i++)
             {
                 r1 = s->ipito + slope*i;
-                ip = r1 + 0.5f;
+                ip = (int32_t) (r1 + 0.5f);
                 if (uvpit != 0.0f)
-                    ip = uvpit;
+                    ip = (int32_t) uvpit;
                 if (ip <= i - jused)
                 {
                     ++(*nout);
@@ -428,14 +438,14 @@ static int pitsyn(lpc10_decode_state_t *s,
                     prop = (jused - ip/2)/(float) lsamp;
                     for (j = 0;  j < LPC10_ORDER;  j++)
                     {
-                        alro = log((s->rco[j] + 1)/(1 - s->rco[j]));
-                        alrn = log((rc[j] + 1)/(1 - rc[j]));
+                        alro = logf((s->rco[j] + 1)/(1 - s->rco[j]));
+                        alrn = logf((rc[j] + 1)/(1 - rc[j]));
                         xxy = alro + prop*(alrn - alro);
-                        xxy = exp(xxy);
-                        rci[j + *nout*rci_dim1 + 1] = (xxy - 1)/(xxy + 1);
+                        xxy = expf(xxy);
+                        rci[j + *nout*rci_dim1 + 1] = (xxy - 1.0f)/(xxy + 1.0f);
                     }
-                    rmsi[*nout - 1] = log(s->rmso) + prop*(log(*rms) - log(s->rmso));
-                    rmsi[*nout - 1] = exp(rmsi[*nout - 1]);
+                    rmsi[*nout - 1] = logf(s->rmso) + prop*(logf(*rms) - logf(s->rmso));
+                    rmsi[*nout - 1] = expf(rmsi[*nout - 1]);
                 }
             }
             if (vflag != 1)
@@ -491,7 +501,7 @@ static void deemp(lpc10_decode_state_t *s, float x[], int len)
 /*- End of function --------------------------------------------------------*/
 
 /* Convert reflection coefficients to predictor coefficients */
-static float irc2pc(float rc[], float pc[], float gprime)
+static float reflection_coeffs_to_predictor_coeffs(float rc[], float pc[], float gprime)
 {
     float temp[10];
     float g2pass;
@@ -501,7 +511,7 @@ static float irc2pc(float rc[], float pc[], float gprime)
     g2pass = 1.0f;
     for (i = 0;  i < LPC10_ORDER;  i++)
         g2pass *= 1.0f - rc[i]*rc[i];
-    g2pass = gprime*sqrt(g2pass);
+    g2pass = gprime*sqrtf(g2pass);
     pc[0] = rc[0];
     for (i = 1;  i < LPC10_ORDER;  i++)
     {
@@ -523,13 +533,12 @@ static int synths(lpc10_decode_state_t *s,
                   float speech[])
 {
     int32_t i1;
-    float r1;
-    float rmsi[16];
     int32_t ivuv[16];
     int32_t ipiti[16];
     int32_t nout;
     int32_t i;
     int32_t j;
+    float rmsi[16];
     float ratio;
     float g2pass;
     float pc[10];
@@ -538,17 +547,14 @@ static int synths(lpc10_decode_state_t *s,
     i1 = min(*pitch, 156);
     *pitch = max(i1, 20);
     for (i = 0;  i < LPC10_ORDER;  i++)
-    {
-        r1 = min(rc[i], 0.99f);
-        rc[i] = max(r1, -0.99f);
-    }
+        rc[i] = max(min(rc[i], 0.99f), -0.99f);
     pitsyn(s, voice, pitch, rms, rc, ivuv, ipiti, rmsi, rci, &nout, &ratio);
     if (nout > 0)
     {
         for (j = 0;  j < nout;  j++)
         {
             /* Add synthesized speech for pitch period J to the end of s->buf. */
-            g2pass = irc2pc(&rci[j*10], pc, 0.7f);
+            g2pass = reflection_coeffs_to_predictor_coeffs(&rci[j*10], pc, 0.7f);
             bsynz(s, pc, ipiti[j], &ivuv[j], &s->buf[s->buflen], rmsi[j], ratio, g2pass);
             deemp(s, &s->buf[s->buflen], ipiti[j]);
             s->buflen += ipiti[j];
@@ -895,7 +901,7 @@ static void decode(lpc10_decode_state_t *s,
                     s->drc[4 - i][1] = iout;
                 }
                 /* Determine error rate */
-                s->erate = s->erate*0.96875f + errcnt*102.0f;
+                s->erate = (int32_t) (s->erate*0.96875f + errcnt*102.0f);
             }
             /* Get unsmoothed RMS, RC's, and PITCH */
             t->irms = s->drms[1];
@@ -983,7 +989,7 @@ static void decode(lpc10_decode_state_t *s,
     {
         ishift = 15 - nbit[i];
         i2 = t->irc[i]*pow_ii(2, ishift) + qb[i - 2];
-        t->irc[i] = i2*descl[i - 2] + deadd[i - 2];
+        t->irc[i] = (int32_t) (i2*descl[i - 2] + deadd[i - 2]);
     }
     /* Scale RMS and RC's to floats */
     *rms = (float) t->irms;
@@ -1007,7 +1013,7 @@ lpc10_decode_state_t *lpc10_decode_init(lpc10_decode_state_t *s, int error_corre
 
     if (s == NULL)
     {
-        if ((s = (lpc10_decode_state_t *) malloc(sizeof(lpc10_decode_state_t))) == NULL)
+        if ((s = (lpc10_decode_state_t *) malloc(sizeof(*s))) == NULL)
             return NULL;
     }
 
@@ -1074,7 +1080,7 @@ int lpc10_decode_release(lpc10_decode_state_t *s)
 }
 /*- End of function --------------------------------------------------------*/
 
-int lpc10_decode(lpc10_decode_state_t *s, int16_t amp[], const uint8_t code[], int quant)
+int lpc10_decode(lpc10_decode_state_t *s, int16_t amp[], const uint8_t code[], int len)
 {
     int voice[2];
     int32_t pitch;
@@ -1084,18 +1090,21 @@ int lpc10_decode(lpc10_decode_state_t *s, int16_t amp[], const uint8_t code[], i
     float rms;
     int i;
     int j;
+    int base;
 
-    /* Decode 54 bits to LPC10_SAMPLES_PER_FRAME speech samples. */
-    for (i = 0;  i < quant;  i++)
+    /* Decode 54 bits in 7 bytes to LPC10_SAMPLES_PER_FRAME speech samples. */
+    len /= 7;
+    for (i = 0;  i < len;  i++)
     {
         lpc10_unpack(&frame, &code[i*7]);
         decode(s, &frame, voice, &pitch, &rms, rc);
         synths(s, voice, &pitch, &rms, rc, speech);
+        base = i*LPC10_SAMPLES_PER_FRAME;
         for (j = 0;  j < LPC10_SAMPLES_PER_FRAME;  j++)
-            amp[i*LPC10_SAMPLES_PER_FRAME + j] = lrintf(32768.0f*speech[j]);
+            amp[base + j] = (int16_t) lrintf(32768.0f*speech[j]);
     }
 
-    return quant*LPC10_SAMPLES_PER_FRAME;
+    return len*LPC10_SAMPLES_PER_FRAME;
 }
 /*- End of function --------------------------------------------------------*/
 /*- End of file ------------------------------------------------------------*/
