@@ -11,24 +11,24 @@
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 2, as
- * published by the Free Software Foundation.
+ * it under the terms of the GNU Lesser General Public License version 2.1,
+ * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: noise.c,v 1.15 2006/10/24 13:45:26 steveu Exp $
+ * $Id: noise.c,v 1.23 2008/07/02 14:48:25 steveu Exp $
  */
 
 /*! \file */
 
-#ifdef HAVE_CONFIG_H
+#if defined(HAVE_CONFIG_H)
 #include <config.h>
 #endif
 
@@ -36,7 +36,13 @@
 #include <stdio.h>
 #include <inttypes.h>
 #include <memory.h>
+#include "floating_fudge.h"
+#if defined(HAVE_TGMATH_H)
 #include <tgmath.h>
+#endif
+#if defined(HAVE_MATH_H)
+#include <math.h>
+#endif
 
 #include "spandsp/telephony.h"
 #include "spandsp/dc_restore.h"
@@ -81,6 +87,11 @@ noise_state_t *noise_init_dbov(noise_state_t *s, int seed, float level, int clas
 {
     float rms;
 
+    if (s == NULL)
+    {
+        if ((s = (noise_state_t *) malloc(sizeof(*s))) == NULL)
+            return NULL;
+    }
     memset(s, 0, sizeof(*s));
     s->rndnum = (uint32_t) seed;
     rms = 32768.0f*powf(10.0f, level/20.0f);
@@ -92,10 +103,10 @@ noise_state_t *noise_init_dbov(noise_state_t *s, int seed, float level, int clas
         s->quality = quality;
     if (class_of_noise == NOISE_CLASS_HOTH)
     {
-        /* Allow for the gain of the filtering */
-        rms *= 1.043;
+        /* Allow for the gain of the filter */
+        rms *= 1.043f;
     }
-    s->rms = (int32_t) (rms*sqrt(12.0/quality));
+    s->rms = (int32_t) (rms*sqrtf(12.0f/quality));
     s->class_of_noise = class_of_noise;
     return s;
 }
