@@ -10,19 +10,19 @@
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 2, as
- * published by the Free Software Foundation.
+ * it under the terms of the GNU Lesser General Public License version 2.1,
+ * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: lpc10.h,v 1.13 2007/04/08 08:16:18 steveu Exp $
+ * $Id: lpc10.h,v 1.17 2008/04/17 14:27:00 steveu Exp $
  */
 
 #if !defined(_SPANDSP_LPC10_H_)
@@ -184,9 +184,10 @@ int lpc10_encode_release(lpc10_encode_state_t *s);
     \param s The LPC10e context.
     \param ima_data The LPC10e data produced.
     \param amp The audio sample buffer.
-    \param len The number of samples in the buffer.
+    \param len The number of samples in the buffer. This must be a multiple of 180, as
+           this is the number of samples on a frame.
     \return The number of bytes of LPC10e data produced. */
-int lpc10_encode(lpc10_encode_state_t *s, uint8_t code[], const int16_t amp[], int quant);
+int lpc10_encode(lpc10_encode_state_t *s, uint8_t code[], const int16_t amp[], int len);
 
 /*! Initialise an LPC10e decode context.
     \param s The LPC10e context
@@ -200,9 +201,10 @@ int lpc10_decode_release(lpc10_decode_state_t *s);
     \param s The LPC10e context.
     \param amp The audio sample buffer.
     \param code The LPC10e data.
-    \param quant The number of frames of LPC10e data to be decoded.
+    \param len The number of bytes of LPC10e data to be decoded. This must be a multiple of 7,
+           as each frame is packed into 7 bytes.
     \return The number of samples returned. */
-int lpc10_decode(lpc10_decode_state_t *s, int16_t amp[], const uint8_t code[], int quant);
+int lpc10_decode(lpc10_decode_state_t *s, int16_t amp[], const uint8_t code[], int len);
 
 
 #if defined(__cplusplus)
