@@ -10,19 +10,19 @@
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 2, as
- * published by the Free Software Foundation.
+ * it under the terms of the GNU Lesser General Public License version 2.1,
+ * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: power_meter.h,v 1.12 2007/04/10 16:12:20 steveu Exp $
+ * $Id: power_meter.h,v 1.15 2008/04/17 14:27:00 steveu Exp $
  */
 
 #if !defined(_POWER_METER_H_)
@@ -49,15 +49,14 @@ values +/-8031, and this square wave represents 0dBov.  This translates into 6.1
 */
 typedef struct
 {
+    /*! The shift factor, which controls the damping of the power meter. */
     int shift;
-    int low_samples;
-
-    int16_t high_sample;
 
+    /*! The current power reading. */
     int32_t reading;
 } power_meter_t;
 
-#ifdef __cplusplus
+#if defined(__cplusplus)
 extern "C"
 {
 #endif
@@ -113,7 +112,7 @@ int32_t power_meter_level_dbm0(float level);
     \return The equivalent power meter reading. */
 int32_t power_meter_level_dbov(float level);
 
-#ifdef __cplusplus
+#if defined(__cplusplus)
 }
 #endif
 
