@@ -10,19 +10,19 @@
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 2, as
- * published by the Free Software Foundation.
+ * it under the terms of the GNU Lesser General Public License version 2.1,
+ * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: schedule.h,v 1.12 2007/04/08 08:16:18 steveu Exp $
+ * $Id: schedule.h,v 1.16 2008/04/17 14:27:00 steveu Exp $
  */
 
 /*! \file */
@@ -42,6 +42,7 @@ typedef struct span_sched_state_s span_sched_state_t;
 
 typedef void (*span_sched_callback_func_t)(span_sched_state_t *s, void *user_data);
 
+/*! A scheduled event entry. */
 typedef struct
 {
     uint64_t when;
@@ -49,6 +50,7 @@ typedef struct
     void *user_data;
 } span_sched_t;
 
+/*! A scheduled event queue. */
 struct span_sched_state_s
 {
     uint64_t ticker;
@@ -66,8 +68,8 @@ extern "C"
 uint64_t span_schedule_next(span_sched_state_t *s);
 uint64_t span_schedule_time(span_sched_state_t *s);
 
-int span_schedule_event(span_sched_state_t *s, int ms, void (*function)(span_sched_state_t *s, void *data), void *user_data);
-void span_schedule_update(span_sched_state_t *s, int samples);
+int span_schedule_event(span_sched_state_t *s, int us, span_sched_callback_func_t function, void *user_data);
+void span_schedule_update(span_sched_state_t *s, int us);
 void span_schedule_del(span_sched_state_t *s, int id);
 
 span_sched_state_t *span_schedule_init(span_sched_state_t *s);
