@@ -10,19 +10,19 @@
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 2, as
- * published by the Free Software Foundation.
+ * it under the terms of the GNU Lesser General Public License version 2.1,
+ * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: t38_terminal.h,v 1.27 2007/12/14 13:41:17 steveu Exp $
+ * $Id: t38_terminal.h,v 1.34 2008/07/25 13:56:54 steveu Exp $
  */
 
 /*! \file */
@@ -39,32 +39,54 @@
 /* Make sure the HDLC frame buffers are big enough for ECM frames. */
 #define T38_MAX_HDLC_LEN        260
 
-/*!
-    T.38 terminal state.
-*/
 typedef struct
 {
-    /*! Core T.38 support */
+    /*! \brief Core T.38 IFP support */
     t38_core_state_t t38;
 
-    /*! \brief Use (actually allow time for) talker echo protection when transmitting. */
-    int use_tep;    
+    /*! \brief Internet Aware FAX mode bit mask. */
+    int iaf;
 
-    /*! \brief HDLC transmit buffer */
-    uint8_t tx_buf[T38_MAX_HDLC_LEN];
-    /*! \brief The length of the contents of the HDLC transmit buffer */
-    int tx_len;
-    /*! \brief Current pointer within the contents of the HDLC transmit buffer */
-    int tx_ptr;
-
-    /*! \brief HDLC receive buffer */
-    uint8_t rx_buf[T38_MAX_HDLC_LEN];
-    /*! \brief The length of the contents of the HDLC receive buffer */
-    int rx_len;
+    /*! \brief Required time between T.38 transmissions, in ms. */
+    int ms_per_tx_chunk;
+    /*! \brief Bit fields controlling the way data is packed into chunked for transmission. */
+    int chunking_modes;
 
     /*! \brief The current transmit step being timed */
     int timed_step;
 
+    /*! \brief TRUE is there has been some T.38 data missed (i.e. lost packets) in the current
+               reception period. */
+    int missing_data;
+
+    /*! \brief The number of octets to send in each image packet (non-ECM or ECM) at the current
+               rate and the current specified packet interval. */
+    int octets_per_data_packet;
+
+    struct
+    {
+        /*! \brief HDLC receive buffer */
+        uint8_t buf[T38_MAX_HDLC_LEN];
+        /*! \brief The length of the contents of the HDLC receive buffer */
+        int len;
+    } hdlc_rx;
+
+    struct
+    {
+        /*! \brief HDLC transmit buffer */
+        uint8_t buf[T38_MAX_HDLC_LEN];
+        /*! \brief The length of the contents of the HDLC transmit buffer */
+        int len;
+        /*! \brief Current pointer within the contents of the HDLC transmit buffer */
+        int ptr;
+        /*! \brief The number of extra bits in a fully stuffed version of the
+                   contents of the HDLC transmit buffer. This is needed to accurately
+                   estimate the playout time for this frame, through an analogue modem. */
+        int extra_bits;
+    } tx;
+
+    /*! \brief Counter for trailing bytes, used to flush the far end's modem */
+    int trailer_bytes;
     /*! \brief The next queued tramsit indicator */
     int next_tx_indicator;
     /*! \brief The current T.38 data type being transmitted */
@@ -73,46 +95,31 @@ typedef struct
     /*! \brief TRUE if a carrier is present. Otherwise FALSE. */
     int rx_signal_present;
 
-    /*! \brief The T.30 back-end */
-    t30_state_t t30_state;
-
     /*! \brief The current operating mode of the receiver. */
     int current_rx_type;
     /*! \brief The current operating mode of the transmitter. */
     int current_tx_type;
-    
-    /*! \brief Counter for trailing bytes, used to flush the far end's modem */
-    int trailer_bytes;
-
-    /*! \brief TRUE is there has been some T.38 data missed (i.e. lost packets) */
-    int missing_data;
-
-    /*! \brief The number of octets to send in each image packet (non-ECM or ECM) at the current
-               rate and the current specified packet interval. */
-    int octets_per_data_packet;
-    
-    /*! \brief The time between T.38 transmissions, in ms. */
-    int ms_per_tx_chunk;
-    /*! \brief TRUE if multiple data fields should be merged into a single T.38 IFP packet. */
-    int merge_tx_fields;
-
-    /*! \brief The number of times an indicator packet will be sent. Numbers greater than one
-               will increase reliability for UDP transmission. Zero is valid, to suppress all
-               indicator packets for TCP transmission. */
-    int indicator_tx_count;
-
-    /*! \brief The number of times a data packet which ends transmission will be sent. Numbers
-               greater than one will increase reliability for UDP transmission. Zero is not valid. */
-    int data_end_tx_count;
 
+    /*! \brief Current bit rate. */
+    int bit_rate;
     /*! \brief A "sample" count, used to time events. */
     int32_t samples;
     /*! \brief The value for samples at the next transmission point. */
     int32_t next_tx_samples;
+    /*! \brief The current receive timeout. */
     int32_t timeout_rx_samples;
+} t38_terminal_front_end_state_t;
 
-    /*! \brief Internet Aware FAX mode bit mask. */
-    int iaf;
+/*!
+    T.38 terminal state.
+*/
+typedef struct
+{
+    /*! \brief The T.30 back-end */
+    t30_state_t t30;
+
+    /*! \brief The T.38 front-end */
+    t38_terminal_front_end_state_t t38_fe;
 
     logging_state_t logging;
 } t38_terminal_state_t;
@@ -141,6 +148,13 @@ void t38_terminal_set_tep_mode(t38_terminal_state_t *s, int use_tep);
 */
 void t38_terminal_set_fill_bit_removal(t38_terminal_state_t *s, int remove);
 
+/*! Get a pointer to the T.30 engine associated with a termination mode T.38 context.
+    \brief Get a pointer to the T.30 engine associated with a T.38 context.
+    \param s The T.38 context.
+    \return A pointer to the T.30 context, or NULL.
+*/
+t30_state_t *t38_terminal_get_t30_state(t38_terminal_state_t *s);
+
 /*! \brief Initialise a termination mode T.38 context.
     \param s The T.38 context.
     \param calling_party TRUE if the context is for a calling party. FALSE if the
