@@ -10,32 +10,32 @@
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 2, as
- * published by the Free Software Foundation.
+ * it under the terms of the GNU Lesser General Public License version 2.1,
+ * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: telephony.h,v 1.8 2006/10/24 13:45:28 steveu Exp $
+ * $Id: telephony.h,v 1.12 2008/04/17 14:27:01 steveu Exp $
  */
 
-#if !defined(_TELEPHONY_H_)
-#define _TELEPHONY_H_
+#if !defined(_SPANDSP_TELEPHONY_H_)
+#define _SPANDSP_TELEPHONY_H_
 
 #define SAMPLE_RATE             8000
 
 /* This is based on A-law, but u-law is only 0.03dB different */
-#define DBM0_MAX_POWER          (3.14 + 3.02)
-#define DBM0_MAX_SINE_POWER     (3.14)
+#define DBM0_MAX_POWER          (3.14f + 3.02f)
+#define DBM0_MAX_SINE_POWER     (3.14f)
 /* This is based on the ITU definition of dbOv in G.100.1 */
-#define DBOV_MAX_POWER          (0.0)
-#define DBOV_MAX_SINE_POWER     (-3.02)
+#define DBOV_MAX_POWER          (0.0f)
+#define DBOV_MAX_SINE_POWER     (-3.02f)
 
 /*! \brief A handler for pure receive. The buffer cannot be altered. */
 typedef int (span_rx_handler_t)(void *s, const int16_t amp[], int len);
