@@ -1,8 +1,7 @@
 /*
  * SpanDSP - a series of DSP components for telephony
  *
- * tone_generate.h - General telephony tone generation, and specific
- *                   generation of Bell MF, MFC/R2 and network supervisory tones.
+ * tone_generate.h - General telephony tone generation.
  *
  * Written by Steve Underwood <steveu@coppice.org>
  *
@@ -11,25 +10,25 @@
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 2, as
- * published by the Free Software Foundation.
+ * it under the terms of the GNU Lesser General Public License version 2.1,
+ * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: tone_generate.h,v 1.21 2006/10/24 13:45:28 steveu Exp $
+ * $Id: tone_generate.h,v 1.33 2008/04/17 14:27:01 steveu Exp $
  */
 
 /*! \file */
 
-#if !defined(_TONE_GENERATE_H_)
-#define _TONE_GENERATE_H_
+#if !defined(_SPANDSP_TONE_GENERATE_H_)
+#define _SPANDSP_TONE_GENERATE_H_
 
 /*! \page tone_generation_page Tone generation
 \section tone_generation_page_sec_1 What does it do?
@@ -46,16 +45,19 @@ use an exhaustive test to prove the oscillator is stable under all the
 conditions in which we will use it. 
 */
 
+typedef struct
+{
+    int32_t phase_rate;
+    float gain;
+} tone_gen_tone_descriptor_t;
+
 /*!
     Cadenced dual tone generator descriptor.
 */
 typedef struct
 {
-    int32_t phase_rate[2];
-    float gain[2];
-    
+    tone_gen_tone_descriptor_t tone[4];
     int duration[4];
-
     int repeat;
 } tone_gen_descriptor_t;
 
@@ -65,37 +67,35 @@ typedef struct
 */
 typedef struct
 {
-    int32_t phase_rate[2];
-    float gain[2];
-
-    uint32_t phase[2];
+    tone_gen_tone_descriptor_t tone[4];
 
+    uint32_t phase[4];
     int duration[4];
-    
     int repeat;
 
     int current_section;
     int current_position;
 } tone_gen_state_t;
 
-typedef struct
+#if defined(__cplusplus)
+extern "C"
 {
-    int         f1;         /* First freq */
-    int         f2;         /* Second freq */
-    int8_t      level1;     /* Level of the first freq (dB) */
-    int8_t      level2;     /* Level of the second freq (dB) */
-    uint16_t    on_time1;   /* Tone on time (ms) */
-    uint16_t    off_time1;  /* Minimum post tone silence (ms) */
-    uint16_t    on_time2;   /* Tone on time (ms) */
-    uint16_t    off_time2;  /* Minimum post tone silence (ms) */
-    int8_t      repeat;     /* True if cyclic tone, false for one shot. */
-} cadenced_tone_t;
-
-#ifdef __cplusplus
-extern "C" {
 #endif
 
-void make_tone_descriptor(tone_gen_descriptor_t *desc, cadenced_tone_t *tone);
+/*! Create a tone generator descriptor
+    \brief Create a tone generator descriptor
+    \param s The descriptor
+    \param f1 The first frequency, in Hz
+    \param l1 The level of the first frequency, in dBm0
+    \param f2 0 for no second frequency, a positive number for the second frequency,
+           in Hz, or a negative number for an AM modulation frequency, in Hz
+    \param l2 The level of the second frequency, in dBm0, or the percentage modulation depth
+           for an AM modulated tone.
+    \param d1 x
+    \param d2 x
+    \param d3 x
+    \param d4 x
+    \param repeat x */
 void make_tone_gen_descriptor(tone_gen_descriptor_t *s,
                               int f1,
                               int l1,
@@ -107,10 +107,11 @@ void make_tone_gen_descriptor(tone_gen_descriptor_t *s,
                               int d4,
                               int repeat);
 
-void tone_gen_init(tone_gen_state_t *s, tone_gen_descriptor_t *t);
+tone_gen_state_t *tone_gen_init(tone_gen_state_t *s, tone_gen_descriptor_t *t);
+
 int tone_gen(tone_gen_state_t *s, int16_t amp[], int max_samples);
 
-#ifdef __cplusplus
+#if defined(__cplusplus)
 }
 #endif
 
