@@ -10,19 +10,19 @@
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 2, as
- * published by the Free Software Foundation.
+ * it under the terms of the GNU Lesser General Public License version 2.1,
+ * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: v29tx.h,v 1.26 2007/04/08 08:16:18 steveu Exp $
+ * $Id: v29tx.h,v 1.34 2008/07/16 14:23:48 steveu Exp $
  */
 
 /*! \file */
@@ -106,18 +106,31 @@ typedef struct
     int bit_rate;
     /*! \brief The callback function used to get the next bit to be transmitted. */
     get_bit_func_t get_bit;
-    /*! \brief A user specified opaque pointer passed to the callback function. */
-    void *user_data;
+    /*! \brief A user specified opaque pointer passed to the get_bit function. */
+    void *get_bit_user_data;
+
+    /*! \brief The callback function used to report modem status changes. */
+    modem_tx_status_func_t status_handler;
+    /*! \brief A user specified opaque pointer passed to the status function. */
+    void *status_user_data;
 
     /*! \brief Gain required to achieve the specified output power, not allowing
                for the size of the current constellation. */
     float base_gain;
     /*! \brief Gain required to achieve the specified output power, allowing
                for the size of the current constellation. */
+#if defined(SPANDSP_USE_FIXED_POINT)
+    int32_t gain;
+#else
     float gain;
+#endif
 
     /*! \brief The route raised cosine (RRC) pulse shaping filter buffer. */
+#if defined(SPANDSP_USE_FIXED_POINT)
+    complexi16_t rrc_filter[2*V29_TX_FILTER_STEPS];
+#else
     complexf_t rrc_filter[2*V29_TX_FILTER_STEPS];
+#endif
     /*! \brief Current offset into the RRC pulse shaping filter buffer. */
     int rrc_filter_step;
 
@@ -163,26 +176,26 @@ void v29_tx_power(v29_tx_state_t *s, float power);
     use of the context, to initialise its contents.
     \brief Initialise a V.29 modem transmit context.
     \param s The modem context.
-    \param rate The bit rate of the modem. Valid values are 4800, 7200 and 9600.
+    \param bit_rate The bit rate of the modem. Valid values are 4800, 7200 and 9600.
     \param tep TRUE is the optional TEP tone is to be transmitted.
     \param get_bit The callback routine used to get the data to be transmitted.
     \param user_data An opaque pointer.
     \return A pointer to the modem context, or NULL if there was a problem. */
-v29_tx_state_t *v29_tx_init(v29_tx_state_t *s, int rate, int tep, get_bit_func_t get_bit, void *user_data);
+v29_tx_state_t *v29_tx_init(v29_tx_state_t *s, int bit_rate, int tep, get_bit_func_t get_bit, void *user_data);
 
 /*! Reinitialise an existing V.29 modem transmit context, so it may be reused.
     \brief Reinitialise an existing V.29 modem transmit context.
     \param s The modem context.
-    \param rate The bit rate of the modem. Valid values are 4800, 7200 and 9600.
+    \param bit_rate The bit rate of the modem. Valid values are 4800, 7200 and 9600.
     \param tep TRUE is the optional TEP tone is to be transmitted.
     \return 0 for OK, -1 for bad parameter */
-int v29_tx_restart(v29_tx_state_t *s, int rate, int tep);
+int v29_tx_restart(v29_tx_state_t *s, int bit_rate, int tep);
 
-/*! Release a V.29 modem transmit context.
-    \brief Release a V.29 modem transmit context.
+/*! Free a V.29 modem transmit context.
+    \brief Free a V.29 modem transmit context.
     \param s The modem context.
     \return 0 for OK */
-int v29_tx_release(v29_tx_state_t *s);
+int v29_tx_free(v29_tx_state_t *s);
 
 /*! Change the get_bit function associated with a V.29 modem transmit context.
     \brief Change the get_bit function associated with a V.29 modem transmit context.
@@ -191,6 +204,13 @@ int v29_tx_release(v29_tx_state_t *s);
     \param user_data An opaque pointer. */
 void v29_tx_set_get_bit(v29_tx_state_t *s, get_bit_func_t get_bit, void *user_data);
 
+/*! Change the modem status report function associated with a V.29 modem transmit context.
+    \brief Change the modem status report function associated with a V.29 modem transmit context.
+    \param s The modem context.
+    \param handler The callback routine used to report modem status changes.
+    \param user_data An opaque pointer. */
+void v29_tx_set_modem_status_handler(v29_tx_state_t *s, modem_tx_status_func_t handler, void *user_data);
+
 /*! Generate a block of V.29 modem audio samples.
     \brief Generate a block of V.29 modem audio samples.
     \param s The modem context.
