@@ -10,24 +10,24 @@
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 2, as
- * published by the Free Software Foundation.
+ * it under the terms of the GNU Lesser General Public License version 2.1,
+ * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: time_scale.c,v 1.14 2006/10/24 13:45:27 steveu Exp $
+ * $Id: time_scale.c,v 1.22 2008/07/02 14:48:26 steveu Exp $
  */
 
 /*! \file */
 
-#ifdef HAVE_CONFIG_H
+#if defined(HAVE_CONFIG_H)
 #include <config.h>
 #endif
 
@@ -38,7 +38,13 @@
 #include <fcntl.h>
 #include <time.h>
 #include <limits.h>
+#include "floating_fudge.h"
+#if defined(HAVE_TGMATH_H)
 #include <tgmath.h>
+#endif
+#if defined(HAVE_MATH_H)
+#include <math.h>
+#endif
 
 #include "spandsp/telephony.h"
 #include "spandsp/time_scale.h"
@@ -76,38 +82,38 @@ static __inline__ int amdf_pitch(int min_pitch, int max_pitch, int16_t amp[], in
 static __inline__ void overlap_add(int16_t amp1[], int16_t amp2[], int len)
 {
     int i;
-    double weight;
-    double step;
+    float weight;
+    float step;
     
-    step = 1.0/len;
-    weight = 0.0;
+    step = 1.0f/len;
+    weight = 0.0f;
     for (i = 0;  i < len;  i++)
     {
         /* TODO: saturate */
-        amp2[i] = (int16_t) ((double) amp1[i]*(1.0 - weight) + (double) amp2[i]*weight);
+        amp2[i] = (int16_t) ((float) amp1[i]*(1.0f - weight) + (float) amp2[i]*weight);
         weight += step;
     }
 }
 /*- End of function --------------------------------------------------------*/
 
-int time_scale_rate(time_scale_t *s, float rate)
+int time_scale_rate(time_scale_state_t *s, float rate)
 {
-    if (rate <= 0.0)
+    if (rate <= 0.0f)
         return -1;
     /*endif*/
-    if (rate >= 0.99  &&  rate <= 1.01)
+    if (rate >= 0.99f  &&  rate <= 1.01f)
     {
         /* Treat rate close to normal speed as exactly normal speed, and
            avoid divide by zero, and other numerical problems. */
-        rate = 1.0;
+        rate = 1.0f;
     }
-    else if (rate < 1.0)
+    else if (rate < 1.0f)
     {
-        s->rcomp = rate/(1.0 - rate);
+        s->rcomp = rate/(1.0f - rate);
     }
     else
     {
-        s->rcomp = 1.0/(rate - 1.0);
+        s->rcomp = 1.0f/(rate - 1.0f);
     }
     /*endif*/
     s->rate = rate;
@@ -115,19 +121,41 @@ int time_scale_rate(time_scale_t *s, float rate)
 }
 /*- End of function --------------------------------------------------------*/
 
-int time_scale_init(time_scale_t *s, float rate)
+time_scale_state_t *time_scale_init(time_scale_state_t *s, float rate)
 {
-    if (time_scale_rate(s, rate))
-        return -1;
+    int alloced;
+    
+    alloced = FALSE;
+    if (s == NULL)
+    {
+        if ((s = (time_scale_state_t *) malloc(sizeof (*s))) == NULL)
+            return  NULL;
+        /*endif*/
+        alloced = TRUE;
+    }
     /*endif*/
-    s->rate_nudge = 0.0;
+    if (time_scale_rate(s, rate))
+    {
+        if (alloced)
+            free(s);
+        return NULL;
+    }
+    /*endif*/
+    s->rate_nudge = 0.0f;
     s->fill = 0;
     s->lcp = 0;
+    return s;
+}
+/*- End of function --------------------------------------------------------*/
+
+int time_scale_free(time_scale_state_t *s)
+{
+    free(s);
     return 0;
 }
 /*- End of function --------------------------------------------------------*/
 
-int time_scale(time_scale_t *s, int16_t out[], int16_t in[], int len)
+int time_scale(time_scale_state_t *s, int16_t out[], int16_t in[], int len)
 {
     double lcpf;
     int pitch;
@@ -185,7 +213,7 @@ int time_scale(time_scale_t *s, int16_t out[], int16_t in[], int len)
             in_len += s->lcp;
             s->lcp = 0;
         }
-        if (s->rate == 1.0)
+        if (s->rate == 1.0f)
         {
             s->lcp = 0x7FFFFFFF;
         }
@@ -197,17 +225,17 @@ int time_scale(time_scale_t *s, int16_t out[], int16_t in[], int len)
             s->lcp = (int) lcpf;
             /* Note that s->lcp and lcpf are not the same, as lcpf has a fractional part, and s->lcp doesn't */
             s->rate_nudge += s->lcp - lcpf;
-            if (s->rate_nudge >= 0.5)
+            if (s->rate_nudge >= 0.5f)
             {
                 s->lcp--;
-                s->rate_nudge -= 1.0;
+                s->rate_nudge -= 1.0f;
             }
-            else if (s->rate_nudge <= -0.5)
+            else if (s->rate_nudge <= -0.5f)
             {
                 s->lcp++;
-                s->rate_nudge += 1.0;
+                s->rate_nudge += 1.0f;
             }
-            if (s->rate < 1.0)
+            if (s->rate < 1.0f)
             {
                 /* Speed up - drop a chunk of data */
                 overlap_add(s->buf, s->buf + pitch, pitch);
