@@ -10,29 +10,30 @@
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 2, as
- * published by the Free Software Foundation.
+ * it under the terms of the GNU Lesser General Public License version 2.1,
+ * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: tone_detect.c,v 1.39 2007/09/07 13:22:25 steveu Exp $
+ * $Id: tone_detect.c,v 1.47 2008/07/02 14:48:26 steveu Exp $
  */
  
 /*! \file tone_detect.h */
 
-#ifdef HAVE_CONFIG_H
+#if defined(HAVE_CONFIG_H)
 #include "config.h"
 #endif
 
 #include <inttypes.h>
 #include <stdlib.h>
+#include "floating_fudge.h"
 #if defined(HAVE_TGMATH_H)
 #include <tgmath.h>
 #endif
@@ -57,8 +58,8 @@
 
 void make_goertzel_descriptor(goertzel_descriptor_t *t, float freq, int samples)
 {
-#if defined(SPANDSP_USE_FIXED_POINT_EXPERIMENTAL)
-    t->fac = 4096.0f*2.0f*cosf(2.0f*M_PI*(freq/(float) SAMPLE_RATE));
+#if defined(SPANDSP_USE_FIXED_POINT)
+    t->fac = 16383.0f*2.0f*cosf(2.0f*M_PI*(freq/(float) SAMPLE_RATE));
 #else
     t->fac = 2.0f*cosf(2.0f*M_PI*(freq/(float) SAMPLE_RATE));
 #endif
@@ -69,22 +70,34 @@ void make_goertzel_descriptor(goertzel_descriptor_t *t, float freq, int samples)
 goertzel_state_t *goertzel_init(goertzel_state_t *s,
                                 goertzel_descriptor_t *t)
 {
-    if (s  ||  (s = malloc(sizeof(goertzel_state_t))))
+    if (s == NULL)
     {
-        s->v2 =
-        s->v3 = 0.0;
-        s->fac = t->fac;
-        s->samples = t->samples;
-        s->current_sample = 0;
+        if ((s = (goertzel_state_t *) malloc(sizeof(*s))) == NULL)
+            return NULL;
     }
+#if defined(SPANDSP_USE_FIXED_POINT)
+    s->v2 =
+    s->v3 = 0;
+#else
+    s->v2 =
+    s->v3 = 0.0f;
+#endif
+    s->fac = t->fac;
+    s->samples = t->samples;
+    s->current_sample = 0;
     return s;
 }
 /*- End of function --------------------------------------------------------*/
 
 void goertzel_reset(goertzel_state_t *s)
 {
+#if defined(SPANDSP_USE_FIXED_POINT)
     s->v2 =
-    s->v3 = 0.0;
+    s->v3 = 0;
+#else
+    s->v2 =
+    s->v3 = 0.0f;
+#endif
     s->current_sample = 0;
 }
 /*- End of function --------------------------------------------------------*/
@@ -94,8 +107,9 @@ int goertzel_update(goertzel_state_t *s,
                     int samples)
 {
     int i;
-#if defined(SPANDSP_USE_FIXED_POINT_EXPERIMENTAL)
-    int32_t v1;
+#if defined(SPANDSP_USE_FIXED_POINT)
+    int16_t x;
+    int16_t v1;
 #else
     float v1;
 #endif
@@ -106,8 +120,13 @@ int goertzel_update(goertzel_state_t *s,
     {
         v1 = s->v2;
         s->v2 = s->v3;
-#if defined(SPANDSP_USE_FIXED_POINT_EXPERIMENTAL)
-        s->v3 = ((s->fac*s->v2) >> 12) - v1 + (amp[i] >> 8);
+#if defined(SPANDSP_USE_FIXED_POINT)
+        x = (((int32_t) s->fac*s->v2) >> 14);
+        /* Scale down the input signal to avoid overflows. 9 bits is enough to
+           monitor the signals of interest with adequate dynamic range and
+           resolution. In telephony we generally only start with 13 or 14 bits,
+           anyway. */
+        s->v3 = x - v1 + (amp[i] >> 7);
 #else
         s->v3 = s->fac*s->v2 - v1 + amp[i];
 #endif
@@ -117,10 +136,16 @@ int goertzel_update(goertzel_state_t *s,
 }
 /*- End of function --------------------------------------------------------*/
 
+#if defined(SPANDSP_USE_FIXED_POINT)
+int32_t goertzel_result(goertzel_state_t *s)
+#else
 float goertzel_result(goertzel_state_t *s)
+#endif
 {
-#if defined(SPANDSP_USE_FIXED_POINT_EXPERIMENTAL)
-    int32_t v1;
+#if defined(SPANDSP_USE_FIXED_POINT)
+    int16_t v1;
+    int32_t x;
+    int32_t y;
 #else
     float v1;
 #endif
@@ -128,18 +153,33 @@ float goertzel_result(goertzel_state_t *s)
     /* Push a zero through the process to finish things off. */
     v1 = s->v2;
     s->v2 = s->v3;
-#if defined(SPANDSP_USE_FIXED_POINT_EXPERIMENTAL)
-    s->v3 = ((s->fac*s->v2) >> 12) - v1;
+#if defined(SPANDSP_USE_FIXED_POINT)
+    x = (((int32_t) s->fac*s->v2) >> 14);
+    s->v3 = x - v1;
 #else
     s->v3 = s->fac*s->v2 - v1;
 #endif
     /* Now calculate the non-recursive side of the filter. */
     /* The result here is not scaled down to allow for the magnification
        effect of the filter (the usual DFT magnification effect). */
-#if defined(SPANDSP_USE_FIXED_POINT_EXPERIMENTAL)
-    return s->v3*s->v3 + s->v2*s->v2 - ((s->v2*s->v3) >> 12)*s->fac;
+#if defined(SPANDSP_USE_FIXED_POINT)
+    x = (int32_t) s->v3*s->v3;
+    y = (int32_t) s->v2*s->v2;
+    x += y;
+    y = ((int32_t) s->v3*s->fac) >> 14;
+    y *= s->v2;
+    x -= y;
+    x <<= 1;
+    goertzel_reset(s);
+    /* The number returned in a floating point build will be 16384^2 times
+       as big as for a fixed point build, due to the 14 bit shifts
+       (or the square of the 7 bit shifts, depending how you look at it). */
+    return x;
 #else
-    return s->v3*s->v3 + s->v2*s->v2 - s->v2*s->v3*s->fac;
+    v1 = s->v3*s->v3 + s->v2*s->v2 - s->v2*s->v3*s->fac;
+    v1 *= 2.0;
+    goertzel_reset(s);
+    return v1;
 #endif
 }
 /*- End of function --------------------------------------------------------*/
@@ -236,6 +276,9 @@ float periodogram_freq_error(const complexf_t *phase_offset, float scale, const
 {
     complexf_t prediction;
 
+    /* Rotate the last result by the expected phasor offset to the current result. Then
+       find the difference between that predicted position, and the actual one. When
+       scaled by the current signal level, this gives us the frequency error. */
     prediction = complex_mulf(last_result, phase_offset);
     return scale*(result->im*prediction.re - result->re*prediction.im)/(result->re*result->re + result->im*result->im);
 }
