@@ -10,24 +10,24 @@
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 2, as
- * published by the Free Software Foundation.
+ * it under the terms of the GNU Lesser General Public License version 2.1,
+ * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: v17tx.c,v 1.54 2008/01/10 14:06:21 steveu Exp $
+ * $Id: v17tx.c,v 1.62 2008/07/16 14:23:47 steveu Exp $
  */
 
 /*! \file */
 
-#ifdef HAVE_CONFIG_H
+#if defined(HAVE_CONFIG_H)
 #include <config.h>
 #endif
 
@@ -35,6 +35,7 @@
 #include <inttypes.h>
 #include <stdlib.h>
 #include <string.h>
+#include "floating_fudge.h"
 #if defined(HAVE_TGMATH_H)
 #include <tgmath.h>
 #endif
@@ -53,6 +54,11 @@
 
 #include "spandsp/v17tx.h"
 
+#if defined(SPANDSP_USE_FIXED_POINT)
+#define SPANDSP_USE_FIXED_POINTx
+#endif
+
+#include "v17tx_constellation_maps.h"
 #if defined(SPANDSP_USE_FIXED_POINT)
 #include "v17tx_fixed_rrc.h"
 #else
@@ -76,278 +82,6 @@
 
 #define V17_BRIDGE_WORD             0x8880
 
-#if defined(SPANDSP_USE_FIXED_POINT)
-static const complexi16_t v17_14400_constellation[128] =
-#else
-static const complexf_t v17_14400_constellation[128] =
-#endif
-{
-    {-8, -3},       /* 0x00 */
-    { 9,  2},       /* 0x01 */
-    { 2, -9},       /* 0x02 */
-    {-3,  8},       /* 0x03 */
-    { 8,  3},       /* 0x04 */
-    {-9, -2},       /* 0x05 */
-    {-2,  9},       /* 0x06 */
-    { 3, -8},       /* 0x07 */
-    {-8,  1},       /* 0x08 */
-    { 9, -2},       /* 0x09 */
-    {-2, -9},       /* 0x0A */
-    { 1,  8},       /* 0x0B */
-    { 8, -1},       /* 0x0C */
-    {-9,  2},       /* 0x0D */
-    { 2,  9},       /* 0x0E */
-    {-1, -8},       /* 0x0F */
-    {-4, -3},       /* 0x10 */
-    { 5,  2},       /* 0x11 */
-    { 2, -5},       /* 0x12 */
-    {-3,  4},       /* 0x13 */
-    { 4,  3},       /* 0x14 */
-    {-5, -2},       /* 0x15 */
-    {-2,  5},       /* 0x16 */
-    { 3, -4},       /* 0x17 */
-    {-4,  1},       /* 0x18 */
-    { 5, -2},       /* 0x19 */
-    {-2, -5},       /* 0x1A */
-    { 1,  4},       /* 0x1B */
-    { 4, -1},       /* 0x1C */
-    {-5,  2},       /* 0x1D */
-    { 2,  5},       /* 0x1E */
-    {-1, -4},       /* 0x1F */
-    { 4, -3},       /* 0x20 */
-    {-3,  2},       /* 0x21 */
-    { 2,  3},       /* 0x22 */
-    {-3, -4},       /* 0x23 */
-    {-4,  3},       /* 0x24 */
-    { 3, -2},       /* 0x25 */
-    {-2, -3},       /* 0x26 */
-    { 3,  4},       /* 0x27 */
-    { 4,  1},       /* 0x28 */
-    {-3, -2},       /* 0x29 */
-    {-2,  3},       /* 0x2A */
-    { 1, -4},       /* 0x2B */
-    {-4, -1},       /* 0x2C */
-    { 3,  2},       /* 0x2D */
-    { 2, -3},       /* 0x2E */
-    {-1,  4},       /* 0x2F */
-    { 0, -3},       /* 0x30 */
-    { 1,  2},       /* 0x31 */
-    { 2, -1},       /* 0x32 */
-    {-3,  0},       /* 0x33 */
-    { 0,  3},       /* 0x34 */
-    {-1, -2},       /* 0x35 */
-    {-2,  1},       /* 0x36 */
-    { 3,  0},       /* 0x37 */
-    { 0,  1},       /* 0x38 */
-    { 1, -2},       /* 0x39 */
-    {-2, -1},       /* 0x3A */
-    { 1,  0},       /* 0x3B */
-    { 0, -1},       /* 0x3C */
-    {-1,  2},       /* 0x3D */
-    { 2,  1},       /* 0x3E */
-    {-1,  0},       /* 0x3F */
-    { 8, -3},       /* 0x40 */
-    {-7,  2},       /* 0x41 */
-    { 2,  7},       /* 0x42 */
-    {-3, -8},       /* 0x43 */
-    {-8,  3},       /* 0x44 */
-    { 7, -2},       /* 0x45 */
-    {-2, -7},       /* 0x46 */
-    { 3,  8},       /* 0x47 */
-    { 8,  1},       /* 0x48 */
-    {-7, -2},       /* 0x49 */
-    {-2,  7},       /* 0x4A */
-    { 1, -8},       /* 0x4B */
-    {-8, -1},       /* 0x4C */
-    { 7,  2},       /* 0x4D */
-    { 2, -7},       /* 0x4E */
-    {-1,  8},       /* 0x4F */
-    {-4, -7},       /* 0x50 */
-    { 5,  6},       /* 0x51 */
-    { 6, -5},       /* 0x52 */
-    {-7,  4},       /* 0x53 */
-    { 4,  7},       /* 0x54 */
-    {-5, -6},       /* 0x55 */
-    {-6,  5},       /* 0x56 */
-    { 7, -4},       /* 0x57 */
-    {-4,  5},       /* 0x58 */
-    { 5, -6},       /* 0x59 */
-    {-6, -5},       /* 0x5A */
-    { 5,  4},       /* 0x5B */
-    { 4, -5},       /* 0x5C */
-    {-5,  6},       /* 0x5D */
-    { 6,  5},       /* 0x5E */
-    {-5, -4},       /* 0x5F */
-    { 4, -7},       /* 0x60 */
-    {-3,  6},       /* 0x61 */
-    { 6,  3},       /* 0x62 */
-    {-7, -4},       /* 0x63 */
-    {-4,  7},       /* 0x64 */
-    { 3, -6},       /* 0x65 */
-    {-6, -3},       /* 0x66 */
-    { 7,  4},       /* 0x67 */
-    { 4,  5},       /* 0x68 */
-    {-3, -6},       /* 0x69 */
-    {-6,  3},       /* 0x6A */
-    { 5, -4},       /* 0x6B */
-    {-4, -5},       /* 0x6C */
-    { 3,  6},       /* 0x6D */
-    { 6, -3},       /* 0x6E */
-    {-5,  4},       /* 0x6F */
-    { 0, -7},       /* 0x70 */
-    { 1,  6},       /* 0x71 */
-    { 6, -1},       /* 0x72 */
-    {-7,  0},       /* 0x73 */
-    { 0,  7},       /* 0x74 */
-    {-1, -6},       /* 0x75 */
-    {-6,  1},       /* 0x76 */
-    { 7,  0},       /* 0x77 */
-    { 0,  5},       /* 0x78 */
-    { 1, -6},       /* 0x79 */
-    {-6, -1},       /* 0x7A */
-    { 5,  0},       /* 0x7B */
-    { 0, -5},       /* 0x7C */
-    {-1,  6},       /* 0x7D */
-    { 6,  1},       /* 0x7E */
-    {-5,  0}        /* 0x7F */
-};
-
-#if defined(SPANDSP_USE_FIXED_POINT)
-static const complexi16_t v17_12000_constellation[64] =
-#else
-static const complexf_t v17_12000_constellation[64] =
-#endif
-{
-    { 7,  1},       /* 0x00 */
-    {-5, -1},       /* 0x01 */
-    {-1,  5},       /* 0x02 */
-    { 1, -7},       /* 0x03 */
-    {-7, -1},       /* 0x04 */
-    { 5,  1},       /* 0x05 */
-    { 1, -5},       /* 0x06 */
-    {-1,  7},       /* 0x07 */
-    { 3, -3},       /* 0x08 */
-    {-1,  3},       /* 0x09 */
-    { 3,  1},       /* 0x0A */
-    {-3, -3},       /* 0x0B */
-    {-3,  3},       /* 0x0C */
-    { 1, -3},       /* 0x0D */
-    {-3, -1},       /* 0x0E */
-    { 3,  3},       /* 0x0F */
-    { 7, -7},       /* 0x10 */
-    {-5,  7},       /* 0x11 */
-    { 7,  5},       /* 0x12 */
-    {-7, -7},       /* 0x13 */
-    {-7,  7},       /* 0x14 */
-    { 5, -7},       /* 0x15 */
-    {-7, -5},       /* 0x16 */
-    { 7,  7},       /* 0x17 */
-    {-1, -7},       /* 0x18 */
-    { 3,  7},       /* 0x19 */
-    { 7, -3},       /* 0x1A */
-    {-7,  1},       /* 0x1B */
-    { 1,  7},       /* 0x1C */
-    {-3, -7},       /* 0x1D */
-    {-7,  3},       /* 0x1E */
-    { 7, -1},       /* 0x1F */
-    { 3,  5},       /* 0x20 */
-    {-1, -5},       /* 0x21 */
-    {-5,  1},       /* 0x22 */
-    { 5, -3},       /* 0x23 */
-    {-3, -5},       /* 0x24 */
-    { 1,  5},       /* 0x25 */
-    { 5, -1},       /* 0x26 */
-    {-5,  3},       /* 0x27 */
-    {-1,  1},       /* 0x28 */
-    { 3, -1},       /* 0x29 */
-    {-1, -3},       /* 0x2A */
-    { 1,  1},       /* 0x2B */
-    { 1, -1},       /* 0x2C */
-    {-3,  1},       /* 0x2D */
-    { 1,  3},       /* 0x2E */
-    {-1, -1},       /* 0x2F */
-    {-5,  5},       /* 0x30 */
-    { 7, -5},       /* 0x31 */
-    {-5, -7},       /* 0x32 */
-    { 5,  5},       /* 0x33 */
-    { 5, -5},       /* 0x34 */
-    {-7,  5},       /* 0x35 */
-    { 5,  7},       /* 0x36 */
-    {-5, -5},       /* 0x37 */
-    {-5, -3},       /* 0x38 */
-    { 7,  3},       /* 0x39 */
-    { 3, -7},       /* 0x3A */
-    {-3,  5},       /* 0x3B */
-    { 5,  3},       /* 0x3C */
-    {-7, -3},       /* 0x3D */
-    {-3,  7},       /* 0x3E */
-    { 3, -5}        /* 0x3F */
-};
-
-#if defined(SPANDSP_USE_FIXED_POINT)
-static const complexi16_t v17_9600_constellation[32] =
-#else
-static const complexf_t v17_9600_constellation[32] =
-#endif
-{
-    {-8,  2},       /* 0x00 */
-    {-6, -4},       /* 0x01 */
-    {-4,  6},       /* 0x02 */
-    { 2,  8},       /* 0x03 */
-    { 8, -2},       /* 0x04 */
-    { 6,  4},       /* 0x05 */
-    { 4, -6},       /* 0x06 */
-    {-2, -8},       /* 0x07 */
-    { 0,  2},       /* 0x08 */
-    {-6,  4},       /* 0x09 */
-    { 4,  6},       /* 0x0A */
-    { 2,  0},       /* 0x0B */
-    { 0, -2},       /* 0x0C */
-    { 6, -4},       /* 0x0D */
-    {-4, -6},       /* 0x0E */
-    {-2,  0},       /* 0x0F */
-    { 0, -6},       /* 0x10 */
-    { 2, -4},       /* 0x11 */
-    {-4, -2},       /* 0x12 */
-    {-6,  0},       /* 0x13 */
-    { 0,  6},       /* 0x14 */
-    {-2,  4},       /* 0x15 */
-    { 4,  2},       /* 0x16 */
-    { 6,  0},       /* 0x17 */
-    { 8,  2},       /* 0x18 */
-    { 2,  4},       /* 0x19 */
-    { 4, -2},       /* 0x1A */
-    { 2, -8},       /* 0x1B */
-    {-8, -2},       /* 0x1C */
-    {-2, -4},       /* 0x1D */
-    {-4,  2},       /* 0x1E */
-    {-2,  8}        /* 0x1F */
-};
-
-#if defined(SPANDSP_USE_FIXED_POINT)
-static const complexi16_t v17_7200_constellation[16] =
-#else
-static const complexf_t v17_7200_constellation[16] =
-#endif
-{
-    { 6, -6},       /* 0x00 */
-    {-2,  6},       /* 0x01 */
-    { 6,  2},       /* 0x02 */
-    {-6, -6},       /* 0x03 */
-    {-6,  6},       /* 0x04 */
-    { 2, -6},       /* 0x05 */
-    {-6, -2},       /* 0x06 */
-    { 6,  6},       /* 0x07 */
-    {-2,  2},       /* 0x08 */
-    { 6, -2},       /* 0x09 */
-    {-2, -6},       /* 0x0A */
-    { 2,  2},       /* 0x0B */
-    { 2, -2},       /* 0x0C */
-    {-6,  2},       /* 0x0D */
-    { 2,  6},       /* 0x0E */
-    {-2, -2}        /* 0x0F */
-};
-
 static __inline__ int scramble(v17_tx_state_t *s, int in_bit)
 {
     int out_bit;
@@ -364,23 +98,6 @@ static __inline__ complexi16_t training_get(v17_tx_state_t *s)
 static __inline__ complexf_t training_get(v17_tx_state_t *s)
 #endif
 {
-#if defined(SPANDSP_USE_FIXED_POINT)
-    static const complexi16_t abcd[4] =
-    {
-        {-6, -2},
-        { 2, -6},
-        { 6,  2},
-        {-2,  6}
-    };
-#else
-    static const complexf_t abcd[4] =
-    {
-        {-6.0f, -2.0f},
-        { 2.0f, -6.0f},
-        { 6.0f,  2.0f},
-        {-2.0f,  6.0f}
-    };
-#endif
     static const int cdba_to_abcd[4] =
     {
         2, 3, 1, 0
@@ -399,7 +116,7 @@ static __inline__ complexf_t training_get(v17_tx_state_t *s)
             if (s->training_step <= V17_TRAINING_SEG_TEP_B)
             {
                 /* Optional segment: Unmodulated carrier (talker echo protection) */
-                return abcd[0];
+                return v17_abcd_constellation[0];
             }
             if (s->training_step <= V17_TRAINING_SEG_1)
             {
@@ -411,7 +128,7 @@ static __inline__ complexf_t training_get(v17_tx_state_t *s)
 #endif
             }
             /* Segment 1: ABAB... */
-            return abcd[(s->training_step & 1) ^ 1];
+            return v17_abcd_constellation[(s->training_step & 1) ^ 1];
         }
         /* Segment 2: CDBA... */
         /* Apply the scrambler */
@@ -423,7 +140,7 @@ static __inline__ complexf_t training_get(v17_tx_state_t *s)
             /* Go straight to the ones test. */
             s->training_step = V17_TRAINING_SEG_4;
         }
-        return abcd[s->constellation_state];
+        return v17_abcd_constellation[s->constellation_state];
     }
     /* Segment 3: Bridge... */
     shift = ((s->training_step - V17_TRAINING_SEG_3 - 1) & 0x7) << 1;
@@ -431,7 +148,7 @@ static __inline__ complexf_t training_get(v17_tx_state_t *s)
     bits = scramble(s, V17_BRIDGE_WORD >> shift);
     bits = (bits << 1) | scramble(s, V17_BRIDGE_WORD >> (shift + 1));
     s->constellation_state = (s->constellation_state + dibit_to_step[bits]) & 3;
-    return abcd[s->constellation_state];
+    return v17_abcd_constellation[s->constellation_state];
 }
 /*- End of function --------------------------------------------------------*/
 
@@ -502,15 +219,22 @@ static __inline__ complexf_t getbaud(v17_tx_state_t *s)
                 return complex_setf(0.0f, 0.0f);
 #endif
             }
+            if (s->training_step == V17_TRAINING_SHUTDOWN_END)
+            {
+                if (s->status_handler)
+                    s->status_handler(s->status_user_data, MODEM_TX_STATUS_SHUTDOWN_COMPLETE);
+            }
         }
     }
     bits = 0;
     for (i = 0;  i < s->bits_per_symbol;  i++)
     {
-        if ((bit = s->current_get_bit(s->user_data)) == PUTBIT_END_OF_DATA)
+        if ((bit = s->current_get_bit(s->get_bit_user_data)) == PUTBIT_END_OF_DATA)
         {
             /* End of real data. Switch to the fake get_bit routine, until we
                have shut down completely. */
+            if (s->status_handler)
+                s->status_handler(s->status_user_data, MODEM_TX_STATUS_DATA_EXHAUSTED);
             s->current_get_bit = fake_get_bit;
             s->in_training = TRUE;
             bit = 1;
@@ -553,8 +277,8 @@ int v17_tx(v17_tx_state_t *s, int16_t amp[], int len)
         x = complex_seti(0, 0);
         for (i = 0;  i < V17_TX_FILTER_STEPS;  i++)
         {
-            x.re += (int32_t) pulseshaper[9 - s->baud_phase][i]*(int32_t) s->rrc_filter[i + s->rrc_filter_step].re;
-            x.im += (int32_t) pulseshaper[9 - s->baud_phase][i]*(int32_t) s->rrc_filter[i + s->rrc_filter_step].im;
+            x.re += (int32_t) tx_pulseshaper[TX_PULSESHAPER_COEFF_SETS - 1 - s->baud_phase][i]*(int32_t) s->rrc_filter[i + s->rrc_filter_step].re;
+            x.im += (int32_t) tx_pulseshaper[TX_PULSESHAPER_COEFF_SETS - 1 - s->baud_phase][i]*(int32_t) s->rrc_filter[i + s->rrc_filter_step].im;
         }
         /* Now create and modulate the carrier */
         x.re >>= 4;
@@ -567,13 +291,13 @@ int v17_tx(v17_tx_state_t *s, int16_t amp[], int len)
         x = complex_setf(0.0f, 0.0f);
         for (i = 0;  i < V17_TX_FILTER_STEPS;  i++)
         {
-            x.re += pulseshaper[9 - s->baud_phase][i]*s->rrc_filter[i + s->rrc_filter_step].re;
-            x.im += pulseshaper[9 - s->baud_phase][i]*s->rrc_filter[i + s->rrc_filter_step].im;
+            x.re += tx_pulseshaper[TX_PULSESHAPER_COEFF_SETS - 1 - s->baud_phase][i]*s->rrc_filter[i + s->rrc_filter_step].re;
+            x.im += tx_pulseshaper[TX_PULSESHAPER_COEFF_SETS - 1 - s->baud_phase][i]*s->rrc_filter[i + s->rrc_filter_step].im;
         }
         /* Now create and modulate the carrier */
         z = dds_complexf(&(s->carrier_phase), s->carrier_phase_rate);
         /* Don't bother saturating. We should never clip. */
-        amp[sample] = (int16_t) rintf((x.re*z.re - x.im*z.im)*s->gain);
+        amp[sample] = (int16_t) lrintf((x.re*z.re - x.im*z.im)*s->gain);
 #endif
     }
     return sample;
@@ -585,9 +309,9 @@ void v17_tx_power(v17_tx_state_t *s, float power)
     /* The constellation design seems to keep the average power the same, regardless
        of which bit rate is in use. */
 #if defined(SPANDSP_USE_FIXED_POINT)
-    s->gain = 0.223f*powf(10.0f, (power - DBM0_MAX_POWER)/20.0f)*16.0f*(32767.0f/30672.52f)*32768.0f/PULSESHAPER_GAIN;
+    s->gain = 0.223f*powf(10.0f, (power - DBM0_MAX_POWER)/20.0f)*16.0f*(32767.0f/30672.52f)*32768.0f/TX_PULSESHAPER_GAIN;
 #else
-    s->gain = 0.223f*powf(10.0f, (power - DBM0_MAX_POWER)/20.0f)*32768.0f/PULSESHAPER_GAIN;
+    s->gain = 0.223f*powf(10.0f, (power - DBM0_MAX_POWER)/20.0f)*32768.0f/TX_PULSESHAPER_GAIN;
 #endif
 }
 /*- End of function --------------------------------------------------------*/
@@ -597,13 +321,20 @@ void v17_tx_set_get_bit(v17_tx_state_t *s, get_bit_func_t get_bit, void *user_da
     if (s->get_bit == s->current_get_bit)
         s->current_get_bit = get_bit;
     s->get_bit = get_bit;
-    s->user_data = user_data;
+    s->get_bit_user_data = user_data;
 }
 /*- End of function --------------------------------------------------------*/
 
-int v17_tx_restart(v17_tx_state_t *s, int rate, int tep, int short_train)
+void v17_tx_set_modem_status_handler(v17_tx_state_t *s, modem_tx_status_func_t handler, void *user_data)
 {
-    switch (rate)
+    s->status_handler = handler;
+    s->status_user_data = user_data;
+}
+/*- End of function --------------------------------------------------------*/
+
+int v17_tx_restart(v17_tx_state_t *s, int bit_rate, int tep, int short_train)
+{
+    switch (bit_rate)
     {
     case 14400:
         s->bits_per_symbol = 6;
@@ -626,7 +357,7 @@ int v17_tx_restart(v17_tx_state_t *s, int rate, int tep, int short_train)
     }
     /* NB: some modems seem to use 3 instead of 1 for long training */
     s->diff = (short_train)  ?  0  :  1;
-    s->bit_rate = rate;
+    s->bit_rate = bit_rate;
 #if defined(SPANDSP_USE_FIXED_POINT)
     memset(s->rrc_filter, 0, sizeof(s->rrc_filter));
 #else
@@ -637,7 +368,7 @@ int v17_tx_restart(v17_tx_state_t *s, int rate, int tep, int short_train)
     s->scramble_reg = 0x2ECDD5;
     s->in_training = TRUE;
     s->short_train = short_train;
-    s->training_step =  (tep)  ?  V17_TRAINING_SEG_TEP_A  :  V17_TRAINING_SEG_1;
+    s->training_step = (tep)  ?  V17_TRAINING_SEG_TEP_A  :  V17_TRAINING_SEG_1;
     s->carrier_phase = 0;
     s->baud_phase = 0;
     s->constellation_state = 0;
@@ -646,7 +377,7 @@ int v17_tx_restart(v17_tx_state_t *s, int rate, int tep, int short_train)
 }
 /*- End of function --------------------------------------------------------*/
 
-v17_tx_state_t *v17_tx_init(v17_tx_state_t *s, int rate, int tep, get_bit_func_t get_bit, void *user_data)
+v17_tx_state_t *v17_tx_init(v17_tx_state_t *s, int bit_rate, int tep, get_bit_func_t get_bit, void *user_data)
 {
     if (s == NULL)
     {
@@ -657,10 +388,10 @@ v17_tx_state_t *v17_tx_init(v17_tx_state_t *s, int rate, int tep, get_bit_func_t
     span_log_init(&s->logging, SPAN_LOG_NONE, NULL);
     span_log_set_protocol(&s->logging, "V.17 TX");
     s->get_bit = get_bit;
-    s->user_data = user_data;
+    s->get_bit_user_data = user_data;
     s->carrier_phase_rate = dds_phase_ratef(CARRIER_NOMINAL_FREQ);
     v17_tx_power(s, -14.0f);
-    v17_tx_restart(s, rate, tep, FALSE);
+    v17_tx_restart(s, bit_rate, tep, FALSE);
     return s;
 }
 /*- End of function --------------------------------------------------------*/
