@@ -1,6 +1,6 @@
-#define PULSESHAPER_4800_GAIN        0.875534f
-#define PULSESHAPER_4800_COEFF_SETS  5
-static const int16_t pulseshaper_4800[PULSESHAPER_4800_COEFF_SETS][9] =
+#define TX_PULSESHAPER_4800_GAIN        0.875534f
+#define TX_PULSESHAPER_4800_COEFF_SETS  5
+static const int16_t tx_pulseshaper_4800[TX_PULSESHAPER_4800_COEFF_SETS][9] =
 {
     {
               58,     /* Filter 0 */
