@@ -1,6 +1,6 @@
-#define PULSESHAPER_4800_GAIN        1.000000f
-#define PULSESHAPER_4800_COEFF_SETS  5
-static const float pulseshaper_4800[PULSESHAPER_4800_COEFF_SETS][9] =
+#define TX_PULSESHAPER_4800_GAIN        1.000000f
+#define TX_PULSESHAPER_4800_COEFF_SETS  5
+static const float tx_pulseshaper_4800[TX_PULSESHAPER_4800_COEFF_SETS][9] =
 {
     {
            0.0020271599f,     /* Filter 0 */
