@@ -10,19 +10,19 @@
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 2, as
- * published by the Free Software Foundation.
+ * it under the terms of the GNU Lesser General Public License version 2.1,
+ * as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: v42bis.c,v 1.26 2006/11/03 13:37:21 steveu Exp $
+ * $Id: v42bis.c,v 1.32 2008/05/13 13:17:25 steveu Exp $
  */
 
 /* THIS IS A WORK IN PROGRESS. IT IS NOT FINISHED. 
@@ -31,7 +31,7 @@
 
 /*! \file */
 
-#ifdef HAVE_CONFIG_H
+#if defined(HAVE_CONFIG_H)
 #include <config.h>
 #endif
 
@@ -71,7 +71,7 @@ enum
     V42BIS_RESET = 2        /* Force reinitialisation */
 };
 
-static __inline__ void push_compressed_octet(v42bis_compress_state_t *ss, int octet)
+static __inline__ void push_compressed_raw_octet(v42bis_compress_state_t *ss, int octet)
 {
     ss->output_buf[ss->output_octet_count++] = (uint8_t) octet;
     if (ss->output_octet_count >= ss->max_len)
@@ -88,7 +88,20 @@ static __inline__ void push_compressed_code(v42bis_compress_state_t *ss, int cod
     ss->output_bit_count += ss->v42bis_parm_c2;
     while (ss->output_bit_count >= 8)
     {
-        push_compressed_octet(ss, ss->output_bit_buffer >> 24);
+        push_compressed_raw_octet(ss, ss->output_bit_buffer >> 24);
+        ss->output_bit_buffer <<= 8;
+        ss->output_bit_count -= 8;
+    }
+}
+/*- End of function --------------------------------------------------------*/
+
+static __inline__ void push_compressed_octet(v42bis_compress_state_t *ss, int code)
+{
+    ss->output_bit_buffer |= code << (32 - 8 - ss->output_bit_count);
+    ss->output_bit_count += 8;
+    while (ss->output_bit_count >= 8)
+    {
+        push_compressed_raw_octet(ss, ss->output_bit_buffer >> 24);
         ss->output_bit_buffer <<= 8;
         ss->output_bit_count -= 8;
     }
@@ -129,7 +142,7 @@ int v42bis_compress(v42bis_state_t *s, const uint8_t *buf, int len)
         {
             /* The leaf exists. Now find it in the table. */
             /* TODO: This is a brute force scan for a match. We need something better. */
-            for (code = 0;  code < s->v42bis_parm_n2;  code++)
+            for (code = 0;  code < ss->v42bis_parm_c3;  code++)
             {
                 if (ss->dict[code].parent_code == ss->string_code  &&  ss->dict[code].node_octet == octet)
                     break;
@@ -144,7 +157,7 @@ int v42bis_compress(v42bis_state_t *s, const uint8_t *buf, int len)
                   created by the last invocation of the string matching procedure, then the
                   next character shall be read and appended to the string and this step
                   repeated. */
-        if (code < s->v42bis_parm_n2  &&  code != ss->latest_code)
+        if (code < ss->v42bis_parm_c3  &&  code != ss->latest_code)
         {
             /* The string was found */
             ss->string_code = code;
@@ -156,7 +169,7 @@ int v42bis_compress(v42bis_state_t *s, const uint8_t *buf, int len)
             if (!ss->transparent)
             {
                 /* 7.4 Encoding - we now have the longest matchable string, and will need to output the code for it. */
-                while (ss->v42bis_parm_c1 >= ss->v42bis_parm_c3)
+                while (ss->v42bis_parm_c1 >= ss->v42bis_parm_c3  &&  ss->v42bis_parm_c3 <= s->v42bis_parm_n2)
                 {
                     /* We need to increase the codeword size */
                     /* 7.4(a) */
@@ -211,53 +224,80 @@ int v42bis_compress(v42bis_state_t *s, const uint8_t *buf, int len)
             /* 7.8 Data compressibility test */
             /* Filter on the balance of what went into the compressor, and what came out */
             ss->compressibility_filter += ((((8*ss->string_length - ss->v42bis_parm_c2) << 20) - ss->compressibility_filter) >> 10);
-            if (ss->transparent)
+            if (ss->compression_mode == V42BIS_COMPRESSION_MODE_DYNAMIC)
             {
-                if (ss->compressibility_filter > 0)
-                    ss->compressibility_persistence++;
-                else
-                    ss->compressibility_persistence = 0;
-                /* 7.8.1 Transition to compressed mode - TODO */
-                if (ss->compressibility_persistence > 1000)
+                /* Work out if it is appropriate to change between transparent and
+                   compressed mode. */
+                if (ss->transparent)
                 {
-//printf("Compress\n");
-#if 0
-                    /* Switch out of transparent now, between codes. We need to send the octet which did not
-                       match, just before switching. */
-                    if (octet == ss->escape_code)
+                    if (ss->compressibility_filter > 0)
                     {
-                        push_compressed_octet(ss, ss->escape_code++);
-                        push_compressed_octet(ss, V42BIS_EID);
+                        if (++ss->compressibility_persistence > 1000)
+                        {
+                            /* Schedule a switch to compressed mode */
+                            ss->change_transparency = -1;
+                            ss->compressibility_persistence = 0;
+                        }
                     }
                     else
                     {
-                        push_compressed_octet(ss, octet);
+                        ss->compressibility_persistence = 0;
+                    }
+                }
+                else
+                {
+                    if (ss->compressibility_filter < 0)
+                    {
+                        if (++ss->compressibility_persistence > 1000)
+                        {
+                            /* Schedule a switch to transparent mode */
+                            ss->change_transparency = 1;
+                            ss->compressibility_persistence = 0;
+                        }
+                    }
+                    else
+                    {
+                        ss->compressibility_persistence = 0;
                     }
-                    push_compressed_octet(ss, ss->escape_code++);
-                    push_compressed_octet(ss, V42BIS_ECM);
-                    ss->transparent = FALSE;
-#endif
-                    ss->compressibility_persistence = 0;
                 }
             }
-            else
+            if (ss->change_transparency)
             {
-                if (ss->compressibility_filter < 0)
-                    ss->compressibility_persistence++;
-                else
-                    ss->compressibility_persistence = 0;
-                /* 7.8.2 Transition to transparent mode - TODO */
-                if (ss->compressibility_persistence > 1000)
+                if (ss->change_transparency < 0)
                 {
-//printf("Transparent\n");
-#if 0
-                    /* Switch into transparent now, between codes, and the unmatched octet should
-                       go out in transparent mode, just below */
-                    push_compressed_code(ss, V42BIS_ETM);
-                    ss->transparent = TRUE;
-#endif
-                    ss->compressibility_persistence = 0;
+                    if (ss->transparent)
+                    {
+                        printf("Going compressed\n");
+                        /* 7.8.1 Transition to compressed mode */
+                        /* Switch out of transparent now, between codes. We need to send the octet which did not
+                        match, just before switching. */
+                        if (octet == ss->escape_code)
+                        {
+                            push_compressed_octet(ss, ss->escape_code++);
+                            push_compressed_octet(ss, V42BIS_EID);
+                        }
+                        else
+                        {
+                            push_compressed_octet(ss, octet);
+                        }
+                        push_compressed_octet(ss, ss->escape_code++);
+                        push_compressed_octet(ss, V42BIS_ECM);
+                        ss->transparent = FALSE;
+                    }
                 }
+                else
+                {
+                    if (!ss->transparent)
+                    {
+                        printf("Going transparent\n");
+                        /* 7.8.2 Transition to transparent mode */
+                        /* Switch into transparent now, between codes, and the unmatched octet should
+                           go out in transparent mode, just below */
+                        push_compressed_code(ss, V42BIS_ETM);
+                        ss->transparent = TRUE;
+                    }
+                }
+                ss->change_transparency = 0;
             }
             /* 7.8.3 Reset function - TODO */
             ss->string_code = octet + V42BIS_N6;
@@ -292,12 +332,12 @@ int v42bis_compress_flush(v42bis_state_t *s)
         /* TODO: We use a positive FLUSH at all times. It is really needed, if the
            previous step resulted in no leftover bits. */
         push_compressed_code(ss, V42BIS_FLUSH);
-        while (ss->output_bit_count > 0)
-        {
-            push_compressed_octet(ss, ss->output_bit_buffer >> 24);
-            ss->output_bit_buffer <<= 8;
-            ss->output_bit_count -= 8;
-        }
+    }
+    while (ss->output_bit_count > 0)
+    {
+        push_compressed_raw_octet(ss, ss->output_bit_buffer >> 24);
+        ss->output_bit_buffer <<= 8;
+        ss->output_bit_count -= 8;
     }
     /* Now push out anything remaining. */
     if (ss->output_octet_count > 0)
@@ -334,6 +374,7 @@ int v42bis_decompress(v42bis_state_t *s, const uint8_t *buf, int len)
     uint8_t *string;
     uint32_t code;
     uint32_t new_code;
+    int code_len;
     v42bis_decompress_state_t *ss;
     uint8_t decode_buf[V42BIS_MAX_STRING_SIZE];
 
@@ -348,156 +389,176 @@ int v42bis_decompress(v42bis_state_t *s, const uint8_t *buf, int len)
         return 0;
     }
     ptr = 0;
+    code_len = (ss->transparent)  ?  8  :  ss->v42bis_parm_c2;
     for (;;)
     {
-        if (ss->transparent)
-        {
-            /* TODO: implement transparent mode */
-            while (ptr < len)
-            {
-                code = buf[ptr++];
-                if (ss->escaped)
-                {
-                    ss->escaped = FALSE;
-                    switch (code)
-                    {
-                    case V42BIS_ECM:
-                        break;
-                    case V42BIS_EID:
-                        ss->transparent = FALSE;
-                        break;
-                    case V42BIS_RESET:
-                        break;
-                    }
-                }
-                else if (code == ss->escape_code)
-                {
-                    ss->escape_code++;
-                    ss->escaped = TRUE;
-                }
-                else
-                {
-                }
-            }
-            if (ss->transparent)
-                break;
-        }
-        /* Gather enough bits for at least one whole code word */
-        while (ss->input_bit_count <= ss->v42bis_parm_c2  &&  ptr < len)
+        /* Fill up the bit buffer. */
+        while (ss->input_bit_count < 32 - 8  &&  ptr < len)
         {
             ss->input_bit_count += 8;
             ss->input_bit_buffer |= (uint32_t) buf[ptr++] << (32 - ss->input_bit_count);
         }
-        if (ss->input_bit_count <= ss->v42bis_parm_c2)
+        if (ss->input_bit_count < code_len)
             break;
-        new_code = ss->input_bit_buffer >> (32 - ss->v42bis_parm_c2);
-        ss->input_bit_count -= ss->v42bis_parm_c2;
-        ss->input_bit_buffer <<= ss->v42bis_parm_c2;
-        if (new_code < V42BIS_N6)
+        new_code = ss->input_bit_buffer >> (32 - code_len);
+        ss->input_bit_count -= code_len;
+        ss->input_bit_buffer <<= code_len;
+        if (ss->transparent)
         {
-            /* We have a control code. */
-            switch (new_code)
+            code = new_code;
+            if (ss->escaped)
             {
-            case V42BIS_ETM:
-                printf("Hit V42BIS_ETM\n");
-                ss->transparent = TRUE;
-                break;
-            case V42BIS_FLUSH:
-                printf("Hit V42BIS_FLUSH\n");
-                v42bis_decompress_flush(s);
-                break;
-            case V42BIS_STEPUP:
-                /* We need to increase the codeword size */
-                printf("Hit V42BIS_STEPUP\n");
-                if (ss->v42bis_parm_c3 >= s->v42bis_parm_n2)
+                ss->escaped = FALSE;
+                if (code == V42BIS_ECM)
                 {
-                    /* Invalid condition */
-                    return -1;
+                    printf("Hit V42BIS_ECM\n");
+                    ss->transparent = FALSE;
+                    code_len = ss->v42bis_parm_c2;
+                }
+                else if (code == V42BIS_EID)
+                {
+                    printf("Hit V42BIS_EID\n");
+                    ss->output_buf[ss->output_octet_count++] = ss->escape_code - 1;
+                    if (ss->output_octet_count >= ss->max_len - s->v42bis_parm_n7)
+                    {
+                        ss->handler(ss->user_data, ss->output_buf, ss->output_octet_count);
+                        ss->output_octet_count = 0;
+                    }
+                }
+                else if (code == V42BIS_RESET)
+                {
+                    printf("Hit V42BIS_RESET\n");
+                }
+                else
+                {
+                    printf("Hit V42BIS_???? - %" PRIu32 "\n", code);
+                }
+            }
+            else if (code == ss->escape_code)
+            {
+                ss->escape_code++;
+                ss->escaped = TRUE;
+            }
+            else
+            {
+                ss->output_buf[ss->output_octet_count++] = (uint8_t) code;
+                if (ss->output_octet_count >= ss->max_len - s->v42bis_parm_n7)
+                {
+                    ss->handler(ss->user_data, ss->output_buf, ss->output_octet_count);
+                    ss->output_octet_count = 0;
                 }
-                ss->v42bis_parm_c2++;
-                ss->v42bis_parm_c3 <<= 1;
-                break;
             }
-            continue;
         }
-        if (ss->first)
+        else
         {
-            ss->first = FALSE;
-            ss->octet = new_code - V42BIS_N6;
-            ss->output_buf[0] = (uint8_t) ss->octet;
-            ss->output_octet_count = 1;
+            if (new_code < V42BIS_N6)
+            {
+                /* We have a control code. */
+                switch (new_code)
+                {
+                case V42BIS_ETM:
+                    printf("Hit V42BIS_ETM\n");
+                    ss->transparent = TRUE;
+                    code_len = 8;
+                    break;
+                case V42BIS_FLUSH:
+                    printf("Hit V42BIS_FLUSH\n");
+                    v42bis_decompress_flush(s);
+                    break;
+                case V42BIS_STEPUP:
+                    /* We need to increase the codeword size */
+                    printf("Hit V42BIS_STEPUP\n");
+                    if (ss->v42bis_parm_c3 >= s->v42bis_parm_n2)
+                    {
+                        /* Invalid condition */
+                        return -1;
+                    }
+                    code_len = ++ss->v42bis_parm_c2;
+                    ss->v42bis_parm_c3 <<= 1;
+                    break;
+                }
+                continue;
+            }
+            if (ss->first)
+            {
+                ss->first = FALSE;
+                ss->octet = new_code - V42BIS_N6;
+                ss->output_buf[0] = (uint8_t) ss->octet;
+                ss->output_octet_count = 1;
+                if (ss->output_octet_count >= ss->max_len - s->v42bis_parm_n7)
+                {
+                    ss->handler(ss->user_data, ss->output_buf, ss->output_octet_count);
+                    ss->output_octet_count = 0;
+                }
+                ss->old_code = new_code;
+                continue;
+            }
+            /* Start at the end of the buffer, and decode backwards */
+            string = &decode_buf[V42BIS_MAX_STRING_SIZE - 1];
+            /* Check the received code is valid. It can't be too big, as we pulled only the expected number
+            of bits from the input stream. It could, however, be unknown. */
+            if (ss->dict[new_code].parent_code == 0xFFFF)
+                return -1;
+            /* Otherwise we do a straight decode of the new code. */
+            code = new_code;
+            /* Trace back through the octets which form the string, and output them. */
+            while (code >= V42BIS_N5)
+            {
+if (code > 4095) {printf("Code is 0x%" PRIu32 "\n", code); exit(2);}
+                *string-- = ss->dict[code].node_octet;
+                code = ss->dict[code].parent_code;
+            }
+            *string = (uint8_t) (code - V42BIS_N6);
+            ss->octet = code - V42BIS_N6;
+            /* Output the decoded string. */
+            this_length = V42BIS_MAX_STRING_SIZE - (int) (string - decode_buf);
+            memcpy(ss->output_buf + ss->output_octet_count, string, this_length);
+            ss->output_octet_count += this_length;
             if (ss->output_octet_count >= ss->max_len - s->v42bis_parm_n7)
             {
                 ss->handler(ss->user_data, ss->output_buf, ss->output_octet_count);
                 ss->output_octet_count = 0;
             }
-            ss->old_code = new_code;
-            continue;
-        }
-        /* Start at the end of the buffer, and decode backwards */
-        string = &decode_buf[V42BIS_MAX_STRING_SIZE - 1];
-        /* Check the received code is valid. It can't be too big, as we pulled only the expected number
-           of bits from the input stream. It could, however, be unknown. */
-        if (ss->dict[new_code].parent_code == 0xFFFF)
-            return -1;
-        /* Otherwise we do a straight decode of the new code. */
-        code = new_code;
-        /* Trace back through the octets which form the string, and output them. */
-        while (code >= V42BIS_N5)
-        {
-            *string-- = ss->dict[code].node_octet;
-            code = ss->dict[code].parent_code;
-        }
-        *string = (uint8_t) (code - V42BIS_N6);
-        ss->octet = code - V42BIS_N6;
-        /* Output the decoded string. */
-        this_length = V42BIS_MAX_STRING_SIZE - (int) (string - decode_buf);
-        memcpy(ss->output_buf + ss->output_octet_count, string, this_length);
-        ss->output_octet_count += this_length;
-        if (ss->output_octet_count >= ss->max_len - s->v42bis_parm_n7)
-        {
-            ss->handler(ss->user_data, ss->output_buf, ss->output_octet_count);
-            ss->output_octet_count = 0;
-        }
-        /* 6.4 Add the string to the dictionary */
-        if (ss->last_length < s->v42bis_parm_n7)
-        {
-            /* 6.4(a) The string does not exceed N7 in length */
-            if (ss->last_old_code != ss->old_code
-                ||
-                ss->last_extra_octet != *string)
+            /* 6.4 Add the string to the dictionary */
+            if (ss->last_length < s->v42bis_parm_n7)
             {
-                /* 6.4(b) The string is not in the table. */
-                ss->dict[ss->old_code].leaves++;
-                /* The new one is definitely a leaf */
-                ss->dict[ss->v42bis_parm_c1].parent_code = (uint16_t) ss->old_code;
-                ss->dict[ss->v42bis_parm_c1].leaves = 0;
-                ss->dict[ss->v42bis_parm_c1].node_octet = (uint8_t) ss->octet;
-                /* 6.5 Recovering a dictionary entry to use next */
-                for (;;)
+                /* 6.4(a) The string does not exceed N7 in length */
+                if (ss->last_old_code != ss->old_code
+                    ||
+                    ss->last_extra_octet != *string)
                 {
-                    /* 6.5(a) and (b) */
-                    if (++ss->v42bis_parm_c1 >= s->v42bis_parm_n2)
-                        ss->v42bis_parm_c1 = V42BIS_N5;
-                    /* 6.5(c) We need to reuse a leaf node */
-                    if (ss->dict[ss->v42bis_parm_c1].leaves)
-                        continue;
-                    /* 6.5(d) This is a leaf node, so re-use it */
-                    /* Possibly make the parent a leaf node again */
-                    if (ss->dict[ss->v42bis_parm_c1].parent_code != 0xFFFF)
-                        ss->dict[ss->dict[ss->v42bis_parm_c1].parent_code].leaves--;
-                    ss->dict[ss->v42bis_parm_c1].parent_code = 0xFFFF;
-                    break;
+                    /* 6.4(b) The string is not in the table. */
+                    ss->dict[ss->old_code].leaves++;
+                    /* The new one is definitely a leaf */
+                    ss->dict[ss->v42bis_parm_c1].parent_code = (uint16_t) ss->old_code;
+                    ss->dict[ss->v42bis_parm_c1].leaves = 0;
+                    ss->dict[ss->v42bis_parm_c1].node_octet = (uint8_t) ss->octet;
+                    /* 6.5 Recovering a dictionary entry to use next */
+                    for (;;)
+                    {
+                        /* 6.5(a) and (b) */
+                        if (++ss->v42bis_parm_c1 >= s->v42bis_parm_n2)
+                            ss->v42bis_parm_c1 = V42BIS_N5;
+                        /* 6.5(c) We need to reuse a leaf node */
+                        if (ss->dict[ss->v42bis_parm_c1].leaves)
+                            continue;
+                        /* 6.5(d) This is a leaf node, so re-use it */
+                        /* Possibly make the parent a leaf node again */
+                        if (ss->dict[ss->v42bis_parm_c1].parent_code != 0xFFFF)
+                            ss->dict[ss->dict[ss->v42bis_parm_c1].parent_code].leaves--;
+                        ss->dict[ss->v42bis_parm_c1].parent_code = 0xFFFF;
+                        break;
+                    }
                 }
             }
-        }
-        /* Record the addition to the dictionary, so we can check for repeat attempts
-           at the next code - see II.4.3 */
-        ss->last_old_code = ss->old_code;
-        ss->last_extra_octet = *string;
+            /* Record the addition to the dictionary, so we can check for repeat attempts
+               at the next code - see II.4.3 */
+            ss->last_old_code = ss->old_code;
+            ss->last_extra_octet = *string;
 
-        ss->old_code = new_code;
-        ss->last_length = this_length;
+            ss->old_code = new_code;
+            ss->last_length = this_length;
+        }
     }
     return 0;
 }
@@ -535,6 +596,21 @@ int v42bis_decompress_dump(v42bis_state_t *s)
 /*- End of function --------------------------------------------------------*/
 #endif
 
+void v42bis_compression_control(v42bis_state_t *s, int mode)
+{
+    s->compress.compression_mode = mode;
+    switch (mode)
+    {
+    case V42BIS_COMPRESSION_MODE_ALWAYS:
+        s->compress.change_transparency = -1;
+        break;
+    case V42BIS_COMPRESSION_MODE_NEVER:
+        s->compress.change_transparency = 1;
+        break;
+    }
+}
+/*- End of function --------------------------------------------------------*/
+
 v42bis_state_t *v42bis_init(v42bis_state_t *s,
                             int negotiated_p0,
                             int negotiated_p1,
@@ -602,6 +678,8 @@ v42bis_state_t *v42bis_init(v42bis_state_t *s,
     s->decompress.last_old_code = 0xFFFFFFFF;
     s->decompress.last_extra_octet = -1;
 
+    s->compress.compression_mode = V42BIS_COMPRESSION_MODE_DYNAMIC;
+
     return s;
 }
 /*- End of function --------------------------------------------------------*/
