@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: adsi_tests.c,v 1.37 2007/11/10 11:14:57 steveu Exp $
+ * $Id: adsi_tests.c,v 1.41 2008/05/13 13:17:25 steveu Exp $
  */
 
 /*! \page adsi_tests_page ADSI tests
@@ -36,7 +36,7 @@ tests, these tests do not include line modelling.
 \section adsi_tests_page_sec_2 How does it work?
 */
 
-#ifdef HAVE_CONFIG_H
+#if defined(HAVE_CONFIG_H)
 #include "config.h"
 #endif
 
