@@ -22,23 +22,20 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: awgn_tests.c,v 1.11 2006/10/24 13:45:28 steveu Exp $
+ * $Id: awgn_tests.c,v 1.17 2008/05/13 13:17:25 steveu Exp $
  */
 
 /*! \page awgn_tests_page AWGN tests
 \section awgn_tests_page_sec_1 What does it do?
 */
 
-#ifdef HAVE_CONFIG_H
+#if defined(HAVE_CONFIG_H)
 #include "config.h"
 #endif
 
-#include <stdio.h>
-#include <inttypes.h>
 #include <stdlib.h>
+#include <stdio.h>
 #include <string.h>
-#include <math.h>
-#include <tiffio.h>
 
 #include "spandsp.h"
 
