@@ -22,24 +22,21 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: dc_restore_tests.c,v 1.17 2006/10/24 13:45:29 steveu Exp $
+ * $Id: dc_restore_tests.c,v 1.23 2008/05/13 13:17:25 steveu Exp $
  */
 
 /*! \page dc_restore_tests_page DC restoration tests
 \section dc_restore_tests_page_sec_1 What does it do?
 */
 
-#ifdef HAVE_CONFIG_H
+#if defined(HAVE_CONFIG_H)
 #include "config.h"
 #endif
 
 #include <stdlib.h>
-#include <inttypes.h>
 #include <stdio.h>
 #include <memory.h>
-#include <tgmath.h>
 #include <time.h>
-#include <tiffio.h>
 
 #include "spandsp.h"
     
