@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: echo_monitor.cpp,v 1.10 2006/10/24 13:22:02 steveu Exp $
+ * $Id: echo_monitor.cpp,v 1.12 2008/05/27 15:08:21 steveu Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -31,6 +31,8 @@
 
 #if defined(HAVE_FL_FL_H)  &&  defined(HAVE_FL_FL_CARTESIAN_H)
 
+#define __STDC_LIMIT_MACROS
+
 #include <inttypes.h>
 #include <stdio.h>
 #include <math.h>
@@ -47,12 +49,11 @@
 #include <FL/Fl.H>
 #include <FL/Fl_Overlay_Window.H>
 #include <FL/Fl_Light_Button.H>
-#include <Fl/Fl_Cartesian.H>
-#include <Fl/Fl_Audio_Meter.H>
+#include <FL/Fl_Cartesian.H>
+#include <FL/Fl_Audio_Meter.H>
 #include <FL/fl_draw.H>
 
-#include "../src/spandsp/complex.h"
-//#include "spandsp.h"
+#include "spandsp.h"
 #include "echo_monitor.h"
 
 Fl_Double_Window *w;
