@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: g711_tests.c,v 1.5 2007/04/10 16:12:20 steveu Exp $
+ * $Id: g711_tests.c,v 1.12 2008/05/13 13:17:25 steveu Exp $
  */
 
 /*! \page g711_tests_page A-law and u-law conversion tests
@@ -31,30 +31,23 @@
 \section g711_tests_page_sec_2 How is it used?
 */
 
-#ifdef HAVE_CONFIG_H
+#if defined(HAVE_CONFIG_H)
 #include "config.h"
 #endif
 
-#include <inttypes.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <fcntl.h>
 #include <string.h>
-#if defined(HAVE_TGMATH_H)
-#include <tgmath.h>
-#endif
-#if defined(HAVE_MATH_H)
-#include <math.h>
-#endif
-#include <assert.h>
 #include <audiofile.h>
-#include <tiffio.h>
 
 #include "spandsp.h"
 
 #define OUT_FILE_NAME   "g711.wav"
 
 int16_t amp[65536];
+uint8_t ulaw_data[65536];
+uint8_t alaw_data[65536];
 
 const uint8_t alaw_1khz_sine[] = {0x34, 0x21, 0x21, 0x34, 0xB4, 0xA1, 0xA1, 0xB4};
 const uint8_t ulaw_1khz_sine[] = {0x1E, 0x0B, 0x0B, 0x1E, 0x9E, 0x8B, 0x8B, 0x9E};
@@ -69,11 +62,16 @@ int main(int argc, char *argv[])
     int block;
     int pre;
     int post;
+    int post_post;
     int alaw_failures;
     int ulaw_failures;
     float worst_alaw;
     float worst_ulaw;
     float tmp;
+    int len;
+    g711_state_t *encode;
+    g711_state_t *transcode;
+    g711_state_t *decode;
     
     if ((filesetup = afNewFileSetup()) == AF_NULL_FILESETUP)
     {
@@ -179,6 +177,33 @@ int main(int argc, char *argv[])
         exit(2);
     }
     
+    printf("Cyclic conversion repeatability tests.\n");
+    /* Find what happens to every possible linear value after a round trip. */
+    for (i = 0;  i < 65536;  i++)
+    {
+        pre = i - 32768;
+        /* Make a round trip */
+        post = alaw_to_linear(linear_to_alaw(pre));
+        /* A second round trip should cause no further change */
+        post_post = alaw_to_linear(linear_to_alaw(post));
+        if (post_post != post)
+        {
+            printf("A-law second round trip mismatch - at %d, %d != %d\n", pre, post, post_post);
+            printf("Tests failed\n");
+            exit(2);
+        }
+        /* Make a round trip */
+        post = ulaw_to_linear(linear_to_ulaw(pre));
+        /* A second round trip should cause no further change */
+        post_post = ulaw_to_linear(linear_to_ulaw(post));
+        if (post_post != post)
+        {
+            printf("u-law round trip mismatch - at %d, %d != %d\n", pre, post, post_post);
+            printf("Tests failed\n");
+            exit(2);
+        }
+    }
+    
     printf("Reference power level tests.\n");
     power_meter_init(&power_meter, 7);
 
@@ -253,6 +278,49 @@ int main(int argc, char *argv[])
         }
     }
     
+    encode = g711_init(NULL, G711_ALAW);
+    transcode = g711_init(NULL, G711_ALAW);
+    decode = g711_init(NULL, G711_ULAW);
+
+    len = 65536;
+    for (i = 0;  i < len;  i++)
+        amp[i] = i - 32768;
+    len = g711_encode(encode, alaw_data, amp, len);
+    len = g711_transcode(transcode, ulaw_data, alaw_data, len);
+    len = g711_decode(decode, amp, ulaw_data, len);
+    if (len != 65536)
+    {
+        printf("Block coding gave the wrong length - %d instead of %d\n", len, 65536);
+        printf("Test failed\n");
+        exit(2);
+    }
+    for (i = 0;  i < len;  i++)
+    {
+        pre = i - 32768;
+        post = amp[i];
+        if (abs(pre) > 140)
+        {
+            tmp = (float) abs(post - pre)/(float) abs(pre);
+            if (tmp > 0.10)
+            {
+                printf("Block: Excessive error at %d (%d)\n", pre, post);
+                exit(2);
+            }
+        }
+        else
+        {
+            /* Small values need different handling for sensible measurement */
+            if (abs(post - pre) > 15)
+            {
+                printf("Block: Excessive error at %d (%d)\n", pre, post);
+                exit(2);
+            }
+        }
+    }
+    g711_release(encode);
+    g711_release(transcode);
+    g711_release(decode);
+
     if (afCloseFile(outhandle))
     {
         fprintf(stderr, "    Cannot close wave file '%s'\n", OUT_FILE_NAME);
