@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: g726_tests.c,v 1.15 2006/10/24 13:45:29 steveu Exp $
+ * $Id: g726_tests.c,v 1.24 2008/05/13 13:17:25 steveu Exp $
  */
 
 /*! \file */
@@ -40,7 +40,7 @@ The speech file should be recorded at 16 bits/sample, 8000 samples/second, and n
 
 \section g726_tests_page_sec_2 How is it used?
 To perform the tests in the G.726 specification you need to obtain the test data files from the
-specification. These are copyright material, and so cannot be distributed with spandsp.
+specification. These are copyright material, and so cannot be distributed with this test software.
 
 The files, containing test vectors, which are supplied with the G.726 specification, should be
 copied to itutests/g726 so the files are arranged in the same directory heirarchy in which they
@@ -51,40 +51,36 @@ are supplied. That is, you should have file names like
     - itutests/g726/DISK2/INPUT/NRM.A
     - itutests/g726/DISK2/INPUT/OVR.A
 
-in your spandsp source tree. The ITU tests can then be run by executing g726_tests without
+in your source tree. The ITU tests can then be run by executing g726_tests without
 any parameters.
 
 To perform a general audio quality test, g726_tests should be run with a parameter specifying
 the required bit rate for compression. The valid parameters are "-16", "-24", "-32", and "-40".
-The test file ../localtests/short_nb_voice.wav will be compressed to the specified bit rate,
+The test file ../test-data/local/short_nb_voice.wav will be compressed to the specified bit rate,
 decompressed, and the resulting audio stored in post_g726.wav.
 */
 
-#ifdef HAVE_CONFIG_H
+#if defined(HAVE_CONFIG_H)
 #include <config.h>
 #endif
 
 #include <stdlib.h>
-#include <unistd.h>
 #include <stdio.h>
 #include <fcntl.h>
-#include <inttypes.h>
+#include <unistd.h>
 #include <memory.h>
-#include <stdlib.h>
-#include <math.h>
-#include <audiofile.h>
 #include <ctype.h>
-#include <tiffio.h>
+#include <audiofile.h>
 
 #include "spandsp.h"
 
 #define BLOCK_LEN           320
 #define MAX_TEST_VECTOR_LEN 40000
 
-#define IN_FILE_NAME    "../localtests/short_nb_voice.wav"
-#define OUT_FILE_NAME   "post_g726.wav"
+#define TESTDATA_DIR    "../test-data/itu/g726/"
 
-#define TESTDATA_DIR    "../itutests/g726/"
+#define IN_FILE_NAME    "../test-data/local/short_nb_voice.wav"
+#define OUT_FILE_NAME   "post_g726.wav"
 
 int16_t outdata[MAX_TEST_VECTOR_LEN];
 uint8_t adpcmdata[MAX_TEST_VECTOR_LEN];
@@ -1274,22 +1270,22 @@ int main(int argc, char *argv[])
     {
         if ((inhandle = afOpenFile(IN_FILE_NAME, "r", 0)) == AF_NULL_FILEHANDLE)
         {
-            printf("    Cannot open wave file '%s'\n", IN_FILE_NAME);
+            fprintf(stderr, "    Cannot open wave file '%s'\n", IN_FILE_NAME);
             exit(2);
         }
         if ((x = afGetFrameSize(inhandle, AF_DEFAULT_TRACK, 1)) != 2.0)
         {
-            printf("    Unexpected frame size in wave file '%s'\n", IN_FILE_NAME);
+            fprintf(stderr, "    Unexpected frame size in wave file '%s'\n", IN_FILE_NAME);
             exit(2);
         }
         if ((x = afGetRate(inhandle, AF_DEFAULT_TRACK)) != (float) SAMPLE_RATE)
         {
-            printf("    Unexpected sample rate in wave file '%s'\n", IN_FILE_NAME);
+            fprintf(stderr, "    Unexpected sample rate in wave file '%s'\n", IN_FILE_NAME);
             exit(2);
         }
         if ((x = afGetChannels(inhandle, AF_DEFAULT_TRACK)) != 1.0)
         {
-            printf("    Unexpected number of channels in wave file '%s'\n", IN_FILE_NAME);
+            fprintf(stderr, "    Unexpected number of channels in wave file '%s'\n", IN_FILE_NAME);
             exit(2);
         }
         if ((filesetup = afNewFileSetup()) == AF_NULL_FILESETUP)
