@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: modem_connect_tones_tests.c,v 1.9 2007/06/28 13:56:10 steveu Exp $
+ * $Id: modem_connect_tones_tests.c,v 1.21 2008/05/14 15:41:25 steveu Exp $
  */
 
 /*! \page modem_connect_tones_tests_page Modem connect tones tests
@@ -30,34 +30,34 @@
 These tests...
 */
 
-#ifdef HAVE_CONFIG_H
+#if defined(HAVE_CONFIG_H)
 #include "config.h"
 #endif
 
 #include <stdlib.h>
-#include <inttypes.h>
 #include <stdio.h>
-#if defined(HAVE_TGMATH_H)
-#include <tgmath.h>
-#endif
-#if defined(HAVE_MATH_H)
-#include <math.h>
-#endif
 #include <fcntl.h>
+#include <unistd.h>
 #include <string.h>
 #include <audiofile.h>
-#include <tiffio.h>
 
 #include "spandsp.h"
 
-#define OUTPUT_FILE_NAME    "modem_connect_tones.wav"
+#define SAMPLES_PER_CHUNK           160
 
-#define MITEL_DIR           "../itutests/mitel/"
-#define BELLCORE_DIR        "../itutests/bellcore/"
+#define OUTPUT_FILE_NAME            "modem_connect_tones.wav"
+
+#define MITEL_DIR                   "../test-data/mitel/"
+#define BELLCORE_DIR                "../test-data/bellcore/"
 
 #define FALSE 0
 #define TRUE (!FALSE)
 
+#define LEVEL_MAX                   0
+#define LEVEL_MIN                   -48
+#define LEVEL_MIN_ACCEPT            -43
+#define LEVEL_MIN_REJECT            -44
+
 const char *bellcore_files[] =
 {
     MITEL_DIR    "mitel-cm7291-talkoff.wav",
@@ -70,32 +70,42 @@ const char *bellcore_files[] =
     ""
 };
 
-#define PERFORM_TEST_1A         (1 << 1)
-#define PERFORM_TEST_1B         (1 << 2)
-#define PERFORM_TEST_1C         (1 << 3)
-#define PERFORM_TEST_1D         (1 << 4)
-#define PERFORM_TEST_2A         (1 << 5)
-#define PERFORM_TEST_2B         (1 << 6)
-#define PERFORM_TEST_2C         (1 << 7)
-#define PERFORM_TEST_3A         (1 << 8)
-#define PERFORM_TEST_3B         (1 << 9)
-#define PERFORM_TEST_4          (1 << 10)
-#define PERFORM_TEST_5          (1 << 11)
+enum
+{
+    PERFORM_TEST_1A = (1 << 1),
+    PERFORM_TEST_1B = (1 << 2),
+    PERFORM_TEST_1C = (1 << 3),
+    PERFORM_TEST_1D = (1 << 4),
+    PERFORM_TEST_2A = (1 << 5),
+    PERFORM_TEST_2B = (1 << 6),
+    PERFORM_TEST_2C = (1 << 7),
+    PERFORM_TEST_3A = (1 << 8),
+    PERFORM_TEST_3B = (1 << 9),
+    PERFORM_TEST_3C = (1 << 10),
+    PERFORM_TEST_4 = (1 << 11),
+    PERFORM_TEST_5 = (1 << 12),
+    PERFORM_TEST_6 = (1 << 13)
+};
 
 int preamble_count = 0;
 int preamble_on_at = -1;
 int preamble_off_at = -1;
+int hits = 0;
+int when = 0;
 
 static int preamble_get_bit(void *user_data)
 {
-    static int bit = 0;
+    static int bit_no = 0;
+    int bit;
     
-    /* Generate a section of 101010... preamble, with a scattering of bit errors.
-       Then generate some random bits, which should not look like preamble. */
+    /* Generate a section of HDLC flag octet preamble. Then generate some random
+       bits, which should not look like preamble. */
     if (++preamble_count < 255)
     {
-        bit ^= 1;
-#if 1
+        bit = (bit_no < 2)  ?  0  :  1;
+        if (++bit_no >= 8)
+            bit_no = 0;
+#if 0
         /* Inject some bad bits */
         if (rand()%15 == 0)
             return bit ^ 1;
@@ -109,9 +119,9 @@ static int preamble_get_bit(void *user_data)
 }
 /*- End of function --------------------------------------------------------*/
 
-static void tone_detected(void *user_data, int on, int level)
+static void preamble_detected(void *user_data, int on, int level, int delay)
 {
-    printf("Preamble declared %s at bit %d\n", (on)  ?  "on"  :  "off", preamble_count);
+    printf("Preamble declared %s at bit %d (%ddBm0)\n", (on)  ?  "on"  :  "off", preamble_count, level);
     if (on)
         preamble_on_at = preamble_count;
     else
@@ -120,19 +130,43 @@ static void tone_detected(void *user_data, int on, int level)
 }
 /*- End of function --------------------------------------------------------*/
 
+static void ced_detected(void *user_data, int on, int level, int delay)
+{
+    printf("FAX CED declared %s at %d (%ddBm0)\n", (on)  ?  "on"  :  "off", when, level);
+    if (on)
+        hits++;
+}
+/*- End of function --------------------------------------------------------*/
+
+static void cng_detected(void *user_data, int on, int level, int delay)
+{
+    printf("FAX CNG declared %s at %d (%ddBm0)\n", (on)  ?  "on"  :  "off", when, level);
+    if (on)
+        hits++;
+}
+/*- End of function --------------------------------------------------------*/
+
+static void ec_dis_detected(void *user_data, int on, int level, int delay)
+{
+    printf("EC disable tone declared %s at %d (%ddBm0)\n", (on)  ?  "on"  :  "off", when, level);
+    if (on)
+        hits++;
+}
+/*- End of function --------------------------------------------------------*/
+
 int main(int argc, char *argv[])
 {
     int i;
     int j;
     int pitch;
     int level;
+    int interval;
     int16_t amp[8000];
     modem_connect_tones_rx_state_t cng_rx;
     modem_connect_tones_rx_state_t ced_rx;
-    modem_connect_tones_rx_state_t ec_dis_rx;
-    modem_connect_tones_tx_state_t ec_dis_tx;
+    modem_connect_tones_rx_state_t ans_pr_rx;
+    modem_connect_tones_tx_state_t modem_tone_tx;
     awgn_state_t chan_noise_source;
-    int hits;
     AFfilehandle inhandle;
     AFfilehandle outhandle;
     AFfilesetup filesetup;
@@ -151,11 +185,28 @@ int main(int argc, char *argv[])
     int level2;
     int max_level2;
     int test_list;
-    int when;
+    int opt;
+    char *decode_test_file;
     fsk_tx_state_t preamble_tx;
 
     test_list = 0;
-    for (i = 1;  i < argc;  i++)
+    decode_test_file = NULL;
+    while ((opt = getopt(argc, argv, "d:")) != -1)
+    {
+        switch (opt)
+        {
+        case 'd':
+            decode_test_file = optarg;
+            break;
+        default:
+            //usage();
+            exit(2);
+            break;
+        }
+    }
+    argc -= optind;
+    argv += optind;
+    for (i = 0;  i < argc;  i++)
     {
         if (strcasecmp(argv[i], "1a") == 0)
             test_list |= PERFORM_TEST_1A;
@@ -175,17 +226,21 @@ int main(int argc, char *argv[])
             test_list |= PERFORM_TEST_3A;
         else if (strcasecmp(argv[i], "3b") == 0)
             test_list |= PERFORM_TEST_3B;
+        else if (strcasecmp(argv[i], "3c") == 0)
+            test_list |= PERFORM_TEST_3C;
         else if (strcasecmp(argv[i], "4") == 0)
             test_list |= PERFORM_TEST_4;
         else if (strcasecmp(argv[i], "5") == 0)
             test_list |= PERFORM_TEST_5;
+        else if (strcasecmp(argv[i], "6") == 0)
+            test_list |= PERFORM_TEST_6;
         else
         {
             fprintf(stderr, "Unknown test '%s' specified\n", argv[i]);
             exit(2);
         }
     }
-    if (test_list == 0)
+    if (decode_test_file == NULL  &&  test_list == 0)
         test_list = 0xFFFFFFFF;
 
     if ((filesetup = afNewFileSetup()) == AF_NULL_FILESETUP)
@@ -207,10 +262,10 @@ int main(int argc, char *argv[])
     if ((test_list & PERFORM_TEST_1A))
     {
         printf("Test 1a: CNG generation to a file\n");
-        modem_connect_tones_tx_init(&ec_dis_tx, MODEM_CONNECT_TONES_FAX_CNG);
-        for (i = 0;  i < 1000;  i++)
+        modem_connect_tones_tx_init(&modem_tone_tx, MODEM_CONNECT_TONES_FAX_CNG);
+        for (i = 0;  i < 20*SAMPLE_RATE;  i += SAMPLES_PER_CHUNK)
         {
-            samples = modem_connect_tones_tx(&ec_dis_tx, amp, 160);
+            samples = modem_connect_tones_tx(&modem_tone_tx, amp, SAMPLES_PER_CHUNK);
             outframes = afWriteFrames(outhandle,
                                       AF_DEFAULT_TRACK,
                                       amp,
@@ -229,10 +284,10 @@ int main(int argc, char *argv[])
     if ((test_list & PERFORM_TEST_1B))
     {
         printf("Test 1b: CED generation to a file\n");
-        modem_connect_tones_tx_init(&ec_dis_tx, MODEM_CONNECT_TONES_FAX_CED);
-        for (i = 0;  i < 1000;  i++)
+        modem_connect_tones_tx_init(&modem_tone_tx, MODEM_CONNECT_TONES_FAX_CED);
+        for (i = 0;  i < 20*SAMPLE_RATE;  i += SAMPLES_PER_CHUNK)
         {
-            samples = modem_connect_tones_tx(&ec_dis_tx, amp, 160);
+            samples = modem_connect_tones_tx(&modem_tone_tx, amp, SAMPLES_PER_CHUNK);
             outframes = afWriteFrames(outhandle,
                                       AF_DEFAULT_TRACK,
                                       amp,
@@ -252,10 +307,10 @@ int main(int argc, char *argv[])
     {
         printf("Test 1c: Modulated EC-disable generation to a file\n");
         /* Some with modulation */
-        modem_connect_tones_tx_init(&ec_dis_tx, MODEM_CONNECT_TONES_EC_DISABLE_MOD);
-        for (i = 0;  i < 1000;  i++)
+        modem_connect_tones_tx_init(&modem_tone_tx, MODEM_CONNECT_TONES_ANSAM);
+        for (i = 0;  i < 20*SAMPLE_RATE;  i += SAMPLES_PER_CHUNK)
         {
-            samples = modem_connect_tones_tx(&ec_dis_tx, amp, 160);
+            samples = modem_connect_tones_tx(&modem_tone_tx, amp, SAMPLES_PER_CHUNK);
             outframes = afWriteFrames(outhandle,
                                       AF_DEFAULT_TRACK,
                                       amp,
@@ -275,10 +330,10 @@ int main(int argc, char *argv[])
     {
         printf("Test 1d: EC-disable generation to a file\n");
         /* Some without modulation */
-        modem_connect_tones_tx_init(&ec_dis_tx, MODEM_CONNECT_TONES_EC_DISABLE);
-        for (i = 0;  i < 1000;  i++)
+        modem_connect_tones_tx_init(&modem_tone_tx, MODEM_CONNECT_TONES_ANS_PR);
+        for (i = 0;  i < 20*SAMPLE_RATE;  i += SAMPLES_PER_CHUNK)
         {
-            samples = modem_connect_tones_tx(&ec_dis_tx, amp, 160);
+            samples = modem_connect_tones_tx(&modem_tone_tx, amp, SAMPLES_PER_CHUNK);
             outframes = afWriteFrames(outhandle,
                                       AF_DEFAULT_TRACK,
                                       amp,
@@ -327,9 +382,9 @@ int main(int argc, char *argv[])
             max_power = 0;
             level2 = 0;
             max_level2 = 0;
-            for (i = 0;  i < 500;  i++)
+            for (i = 0;  i < 10*SAMPLE_RATE;  i += SAMPLES_PER_CHUNK)
             {
-                samples = tone_gen(&tone_tx, amp, 160);
+                samples = tone_gen(&tone_tx, amp, SAMPLES_PER_CHUNK);
                 for (j = 0;  j < samples;  j++)
                 {
                     amp[j] += awgn(&chan_noise_source);
@@ -350,13 +405,13 @@ int main(int argc, char *argv[])
             hit = modem_connect_tones_rx_get(&cng_rx);
             if (pitch < (1100 - 70)  ||  pitch > (1100 + 70))
             {
-                if (hit)
+                if (hit == MODEM_CONNECT_TONES_FAX_CNG)
                     false_hit = TRUE;
                 /*endif*/
             }
             else if (pitch > (1100 - 50)  &&  pitch < (1100 + 50))
             {
-                if (!hit)
+                if (hit != MODEM_CONNECT_TONES_FAX_CNG)
                     false_miss = TRUE;
                 /*endif*/
             }
@@ -372,6 +427,7 @@ int main(int argc, char *argv[])
             exit(2);
         }
         /*endif*/
+        printf("Test passed.\n");
     }
     /*endif*/
     
@@ -396,9 +452,9 @@ int main(int argc, char *argv[])
             tone_gen_init(&tone_tx, &tone_desc);
 
             modem_connect_tones_rx_init(&ced_rx, MODEM_CONNECT_TONES_FAX_CED, NULL, NULL);
-            for (i = 0;  i < 500;  i++)
+            for (i = 0;  i < 10*SAMPLE_RATE;  i += SAMPLES_PER_CHUNK)
             {
-                samples = tone_gen(&tone_tx, amp, 160);
+                samples = tone_gen(&tone_tx, amp, SAMPLES_PER_CHUNK);
                 for (j = 0;  j < samples;  j++)
                     amp[j] += awgn(&chan_noise_source);
                 /*endfor*/
@@ -408,12 +464,12 @@ int main(int argc, char *argv[])
             hit = modem_connect_tones_rx_get(&ced_rx);
             if (pitch < (2100 - 70)  ||  pitch > (2100 + 70))
             {
-                if (hit)
+                if (hit == MODEM_CONNECT_TONES_FAX_CED)
                     false_hit = TRUE;
             }
             else if (pitch > (2100 - 50)  &&  pitch < (2100 + 50))
             {
-                if (!hit)
+                if (hit != MODEM_CONNECT_TONES_FAX_CED)
                     false_miss = TRUE;
             }
             /*endif*/
@@ -427,6 +483,7 @@ int main(int argc, char *argv[])
             exit(2);
         }
         /*endif*/
+        printf("Test passed.\n");
     }
     /*endif*/
 
@@ -439,36 +496,36 @@ int main(int argc, char *argv[])
         for (pitch = 2000;  pitch < 2200;  pitch++)
         {
             /* Use the transmitter to test the receiver */
-            modem_connect_tones_tx_init(&ec_dis_tx, MODEM_CONNECT_TONES_EC_DISABLE);
+            modem_connect_tones_tx_init(&modem_tone_tx, MODEM_CONNECT_TONES_ANS_PR);
             /* Fudge things for the test */
-            ec_dis_tx.tone_phase_rate = dds_phase_rate(pitch);
-            ec_dis_tx.level = dds_scaling_dbm0(-25);
-            modem_connect_tones_rx_init(&ec_dis_rx, MODEM_CONNECT_TONES_EC_DISABLE, NULL, NULL);
-            for (i = 0;  i < 500;  i++)
+            modem_tone_tx.tone_phase_rate = dds_phase_rate(pitch);
+            modem_tone_tx.level = dds_scaling_dbm0(-25);
+            modem_connect_tones_rx_init(&ans_pr_rx, MODEM_CONNECT_TONES_ANS_PR, NULL, NULL);
+            for (i = 0;  i < 10*SAMPLE_RATE;  i += SAMPLES_PER_CHUNK)
             {
-                samples = modem_connect_tones_tx(&ec_dis_tx, amp, 160);
+                samples = modem_connect_tones_tx(&modem_tone_tx, amp, SAMPLES_PER_CHUNK);
                 for (j = 0;  j < samples;  j++)
                     amp[j] += awgn(&chan_noise_source);
                 /*endfor*/
-                modem_connect_tones_rx(&ec_dis_rx, amp, samples);
+                modem_connect_tones_rx(&ans_pr_rx, amp, samples);
             }
             /*endfor*/
-            hit = modem_connect_tones_rx_get(&ec_dis_rx);
+            hit = modem_connect_tones_rx_get(&ans_pr_rx);
             if (pitch < (2100 - 70)  ||  pitch > (2100 + 70))
             {
-                if (hit)
+                if (hit == MODEM_CONNECT_TONES_ANS_PR)
                     false_hit = TRUE;
                 /*endif*/
             }
             else if (pitch > (2100 - 50)  &&  pitch < (2100 + 50))
             {
-                if (!hit)
+                if (hit != MODEM_CONNECT_TONES_ANS_PR)
                     false_miss = TRUE;
                 /*endif*/
             }
             /*endif*/
             if (hit)
-                printf("Detected at %5dHz %12d %12d %d\n", pitch, ec_dis_rx.channel_level, ec_dis_rx.notch_level, hit);
+                printf("Detected at %5dHz %12d %12d %d\n", pitch, ans_pr_rx.channel_level, ans_pr_rx.notch_level, hit);
             /*endif*/
         }
         /*endfor*/
@@ -478,6 +535,7 @@ int main(int argc, char *argv[])
             exit(2);
         }
         /*endif*/
+        printf("Test passed.\n");
     }
     /*endif*/
 
@@ -489,7 +547,7 @@ int main(int argc, char *argv[])
         false_miss = FALSE;
         for (pitch = 1062;  pitch <= 1138;  pitch += 2*38)
         {
-            for (level = 0;  level >= -43;  level--)
+            for (level = LEVEL_MAX;  level >= LEVEL_MIN;  level--)
             {
                 make_tone_gen_descriptor(&tone_desc,
                                          pitch,
@@ -504,9 +562,9 @@ int main(int argc, char *argv[])
                 tone_gen_init(&tone_tx, &tone_desc);
 
                 modem_connect_tones_rx_init(&cng_rx, MODEM_CONNECT_TONES_FAX_CNG, NULL, NULL);
-                for (i = 0;  i < 500;  i++)
+                for (i = 0;  i < 10*SAMPLE_RATE;  i += SAMPLES_PER_CHUNK)
                 {
-                    samples = tone_gen(&tone_tx, amp, 160);
+                    samples = tone_gen(&tone_tx, amp, SAMPLES_PER_CHUNK);
                     for (j = 0;  j < samples;  j++)
                         amp[j] += awgn(&chan_noise_source);
                     /*endfor*/
@@ -514,21 +572,27 @@ int main(int argc, char *argv[])
                 }
                 /*endfor*/
                 hit = modem_connect_tones_rx_get(&cng_rx);
-                if (level < -43)
+                if (level < LEVEL_MIN_REJECT)
                 {
-                    if (hit)
+                    if (hit == MODEM_CONNECT_TONES_FAX_CNG)
+                    {
+                        printf("False hit %d at %ddB\n", hit, level);
                         false_hit = TRUE;
+                    }
                     /*endif*/
                 }
-                else if (level > -43)
+                else if (level > LEVEL_MIN_ACCEPT)
                 {
-                    if (!hit)
+                    if (hit != MODEM_CONNECT_TONES_FAX_CNG)
+                    {
+                        printf("False miss %d at %ddB\n", hit, level);
                         false_miss = TRUE;
+                    }
                     /*endif*/
                 }
                 /*endif*/
                 if (hit)
-                    printf("Detected at %5dHz %ddB %12d %12d %d\n", pitch, level, cng_rx.channel_level, cng_rx.notch_level, hit);
+                    printf("Detected at %5dHz %4ddB %12d %12d %d\n", pitch, level, cng_rx.channel_level, cng_rx.notch_level, hit);
                 /*endif*/
             }
             /*endfor*/
@@ -540,6 +604,7 @@ int main(int argc, char *argv[])
             exit(2);
         }
         /*endif*/
+        printf("Test passed.\n");
     }
     /*endif*/
 
@@ -551,7 +616,7 @@ int main(int argc, char *argv[])
         false_miss = FALSE;
         for (pitch = 2062;  pitch <= 2138;  pitch += 2*38)
         {
-            for (level = 0;  level >= -43;  level--)
+            for (level = LEVEL_MAX;  level >= LEVEL_MIN;  level--)
             {
                 make_tone_gen_descriptor(&tone_desc,
                                          pitch,
@@ -565,9 +630,9 @@ int main(int argc, char *argv[])
                                          FALSE);
                 tone_gen_init(&tone_tx, &tone_desc);
                 modem_connect_tones_rx_init(&ced_rx, MODEM_CONNECT_TONES_FAX_CED, NULL, NULL);
-                for (i = 0;  i < 500;  i++)
+                for (i = 0;  i < 10*SAMPLE_RATE;  i += SAMPLES_PER_CHUNK)
                 {
-                    samples = tone_gen(&tone_tx, amp, 160);
+                    samples = tone_gen(&tone_tx, amp, SAMPLES_PER_CHUNK);
                     for (j = 0;  j < samples;  j++)
                         amp[j] += awgn(&chan_noise_source);
                     /*endfor*/
@@ -575,21 +640,21 @@ int main(int argc, char *argv[])
                 }
                 /*endfor*/
                 hit = modem_connect_tones_rx_get(&ced_rx);
-                if (level < -43)
+                if (level < LEVEL_MIN_REJECT)
                 {
-                    if (hit)
+                    if (hit == MODEM_CONNECT_TONES_FAX_CED)
                         false_hit = TRUE;
                     /*endif*/
                 }
-                else if (level > -43)
+                else if (level > LEVEL_MIN_ACCEPT)
                 {
-                    if (!hit)
+                    if (hit != MODEM_CONNECT_TONES_FAX_CED)
                         false_miss = TRUE;
                     /*endif*/
                 }
                 /*endif*/
                 if (hit)
-                    printf("Detected at %5dHz %ddB %12d %12d %d\n", pitch, level, ced_rx.channel_level, ced_rx.notch_level, hit);
+                    printf("Detected at %5dHz %4ddB %12d %12d %d\n", pitch, level, ced_rx.channel_level, ced_rx.notch_level, hit);
                 /*endif*/
             }
             /*endfor*/
@@ -601,6 +666,63 @@ int main(int argc, char *argv[])
             exit(2);
         }
         /*endif*/
+        printf("Test passed.\n");
+    }
+    /*endif*/
+
+    if ((test_list & PERFORM_TEST_3C))
+    {
+        printf("Test 3c: EC disable detection with level\n");
+        awgn_init_dbm0(&chan_noise_source, 7162534, -60.0f);
+        false_hit = FALSE;
+        false_miss = FALSE;
+        for (pitch = 2062;  pitch <= 2138;  pitch += 2*38)
+        {
+            for (level = LEVEL_MAX;  level >= LEVEL_MIN;  level--)
+            {
+                /* Use the transmitter to test the receiver */
+                modem_connect_tones_tx_init(&modem_tone_tx, MODEM_CONNECT_TONES_ANS_PR);
+                /* Fudge things for the test */
+                modem_tone_tx.tone_phase_rate = dds_phase_rate(pitch);
+                modem_tone_tx.level = dds_scaling_dbm0(level);
+                modem_connect_tones_rx_init(&ans_pr_rx, MODEM_CONNECT_TONES_ANS_PR, NULL, NULL);
+                for (i = 0;  i < 10*SAMPLE_RATE;  i += SAMPLES_PER_CHUNK)
+                {
+                    samples = modem_connect_tones_tx(&modem_tone_tx, amp, SAMPLES_PER_CHUNK);
+                    for (j = 0;  j < samples;  j++)
+                        amp[j] += awgn(&chan_noise_source);
+                    /*endfor*/
+                    modem_connect_tones_rx(&ans_pr_rx, amp, samples);
+                }
+                /*endfor*/
+                hit = modem_connect_tones_rx_get(&ans_pr_rx);
+                if (level < LEVEL_MIN_REJECT)
+                {
+                    if (hit == MODEM_CONNECT_TONES_ANS_PR)
+                        false_hit = TRUE;
+                    /*endif*/
+                }
+                else if (level > LEVEL_MIN_ACCEPT)
+                {
+                    if (hit != MODEM_CONNECT_TONES_ANS_PR)
+                        false_miss = TRUE;
+                    /*endif*/
+                }
+                /*endif*/
+                if (hit)
+                    printf("Detected at %5dHz %4ddB %12d %12d %d\n", pitch, level, ans_pr_rx.channel_level, ans_pr_rx.notch_level, hit);
+                /*endif*/
+            }
+            /*endfor*/
+        }
+        /*endfor*/
+        if (false_hit  ||  false_miss)
+        {
+            printf("Test failed.\n");
+            exit(2);
+        }
+        /*endif*/
+        printf("Test passed.\n");
     }
     /*endif*/
 
@@ -613,37 +735,105 @@ int main(int argc, char *argv[])
         /* Send 255 bits of preamble (0.85s, the minimum specified preamble for T.30), and then
            some random bits. Check the preamble detector comes on, and goes off at reasonable times. */
         fsk_tx_init(&preamble_tx, &preset_fsk_specs[FSK_V21CH2], preamble_get_bit, NULL);
-        modem_connect_tones_rx_init(&ced_rx, MODEM_CONNECT_TONES_FAX_CED, tone_detected, NULL);
-        for (i = 0;  i < 100;  i++)
+        modem_connect_tones_rx_init(&ced_rx, MODEM_CONNECT_TONES_FAX_CED, preamble_detected, NULL);
+        for (i = 0;  i < 2*SAMPLE_RATE;  i += SAMPLES_PER_CHUNK)
         {
-            samples = fsk_tx(&preamble_tx, amp, 160);
+            samples = fsk_tx(&preamble_tx, amp, SAMPLES_PER_CHUNK);
             modem_connect_tones_rx(&ced_rx, amp, samples);
         }
         /*endfor*/
-        if (preamble_on_at < 40  ||  preamble_on_at > 80
+        for (i = 0;  i < SAMPLE_RATE/10;  i += SAMPLES_PER_CHUNK)
+        {
+            memset(amp, 0, sizeof(int16_t)*SAMPLES_PER_CHUNK);
+            modem_connect_tones_rx(&ced_rx, amp, SAMPLES_PER_CHUNK);
+        }
+        /*endfor*/
+        if (preamble_on_at < 40  ||  preamble_on_at > 50
             ||
-            preamble_off_at < (255 + 40)  ||  preamble_off_at > (255 + 80))
+            preamble_off_at < 580  ||  preamble_off_at > 620)
         {
             printf("Test failed.\n");
             exit(2);
         }
         /*endif*/
+        printf("Test passed.\n");
     }
     /*endif*/
 
     if ((test_list & PERFORM_TEST_5))
+    {
+        printf("Test 5: EC disable detection with reversal interval\n");
+        awgn_init_dbm0(&chan_noise_source, 7162534, -60.0f);
+        false_hit = FALSE;
+        false_miss = FALSE;
+        pitch = 2100;
+        level = -15;
+        for (interval = 400;  interval < 500;  interval++)
+        {
+            /* Use the transmitter to test the receiver */
+            modem_connect_tones_tx_init(&modem_tone_tx, MODEM_CONNECT_TONES_ANS_PR);
+            /* Fudge things for the test */
+            modem_tone_tx.tone_phase_rate = dds_phase_rate(pitch);
+            modem_tone_tx.level = dds_scaling_dbm0(level);
+            modem_connect_tones_rx_init(&ans_pr_rx, MODEM_CONNECT_TONES_ANS_PR, NULL, NULL);
+            for (i = 0;  i < 10*SAMPLE_RATE;  i += SAMPLES_PER_CHUNK)
+            {
+                samples = SAMPLES_PER_CHUNK;
+                for (j = 0;  j < samples;  j++)
+                {
+                    if (--modem_tone_tx.hop_timer <= 0)
+                    {
+                        modem_tone_tx.hop_timer = ms_to_samples(interval);
+                        modem_tone_tx.tone_phase += 0x80000000;
+                    }
+                    amp[j] = dds_mod(&modem_tone_tx.tone_phase, modem_tone_tx.tone_phase_rate, modem_tone_tx.level, 0);
+                }
+                for (j = 0;  j < samples;  j++)
+                    amp[j] += awgn(&chan_noise_source);
+                /*endfor*/
+                modem_connect_tones_rx(&ans_pr_rx, amp, samples);
+            }
+            /*endfor*/
+            hit = modem_connect_tones_rx_get(&ans_pr_rx);
+            if (interval < (450 - 25)  ||  interval > (450 + 25))
+            {
+                if (hit == MODEM_CONNECT_TONES_ANS_PR)
+                    false_hit = TRUE;
+            }
+            else if (interval > (450 - 25)  &&  interval < (450 + 25))
+            {
+                if (hit != MODEM_CONNECT_TONES_ANS_PR)
+                    false_miss = TRUE;
+            }
+            /*endif*/
+            if (hit)
+                printf("Detected at %5dHz %4ddB %dms %12d %12d %d\n", pitch, level, interval, ans_pr_rx.channel_level, ans_pr_rx.notch_level, hit);
+            /*endif*/
+        }
+        /*endfor*/
+        if (false_hit  ||  false_miss)
+        {
+            printf("Test failed.\n");
+            exit(2);
+        }
+        /*endif*/
+        printf("Test passed.\n");
+    }
+    /*endif*/
+
+    if ((test_list & PERFORM_TEST_6))
     {
         /* Talk-off test */
-        /* Here we use the BellCore talk off test tapes, intended for DTMF detector
-           testing. Presumably they should also have value here, but I am not sure.
-           If those voice snippets were chosen to be tough on DTMF detectors, they
+        /* Here we use the BellCore and Mitel talk off test tapes, intended for DTMF
+           detector testing. Presumably they should also have value here, but I am not
+           sure. If those voice snippets were chosen to be tough on DTMF detectors, they
            might go easy on detectors looking for different pitches. However, the
            Mitel DTMF test tape is known (the hard way) to exercise 2280Hz tone
            detectors quite well. */
-        printf("Test 5: Talk-off test\n");
+        printf("Test 6: Talk-off test\n");
         modem_connect_tones_rx_init(&cng_rx, MODEM_CONNECT_TONES_FAX_CNG, NULL, NULL);
         modem_connect_tones_rx_init(&ced_rx, MODEM_CONNECT_TONES_FAX_CED, NULL, NULL);
-        modem_connect_tones_rx_init(&ec_dis_rx, MODEM_CONNECT_TONES_EC_DISABLE, NULL, NULL);
+        modem_connect_tones_rx_init(&ans_pr_rx, MODEM_CONNECT_TONES_ANS_PR, NULL, NULL);
         hits = 0;
         for (j = 0;  bellcore_files[j][0];  j++)
         {
@@ -679,8 +869,8 @@ int main(int argc, char *argv[])
                 when++;
                 modem_connect_tones_rx(&cng_rx, amp, frames);
                 modem_connect_tones_rx(&ced_rx, amp, frames);
-                modem_connect_tones_rx(&ec_dis_rx, amp, frames);
-                if (modem_connect_tones_rx_get(&cng_rx))
+                modem_connect_tones_rx(&ans_pr_rx, amp, frames);
+                if (modem_connect_tones_rx_get(&cng_rx) != MODEM_CONNECT_TONES_NONE)
                 {
                     /* This is not a true measure of hits, as there might be more
                        than one in a block of data. However, since the only good
@@ -690,18 +880,18 @@ int main(int argc, char *argv[])
                     modem_connect_tones_rx_init(&cng_rx, MODEM_CONNECT_TONES_FAX_CNG, NULL, NULL);
                 }
                 /*endif*/
-                if (modem_connect_tones_rx_get(&ced_rx))
+                if (modem_connect_tones_rx_get(&ced_rx) != MODEM_CONNECT_TONES_NONE)
                 {
                     printf("Hit CED at %ds\n", when);
                     hits++;
                     modem_connect_tones_rx_init(&ced_rx, MODEM_CONNECT_TONES_FAX_CED, NULL, NULL);
                 }
                 /*endif*/
-                if (modem_connect_tones_rx_get(&ec_dis_rx))
+                if (modem_connect_tones_rx_get(&ans_pr_rx) != MODEM_CONNECT_TONES_NONE)
                 {
                     printf("Hit EC disable at %ds\n", when);
                     hits++;
-                    modem_connect_tones_rx_init(&ec_dis_rx, MODEM_CONNECT_TONES_EC_DISABLE, NULL, NULL);
+                    modem_connect_tones_rx_init(&ans_pr_rx, MODEM_CONNECT_TONES_ANS_PR, NULL, NULL);
                 }
                 /*endif*/
             }
@@ -715,15 +905,66 @@ int main(int argc, char *argv[])
             printf("    File %d gave %d false hits.\n", j + 1, hits);
         }
         /*endfor*/
-        if (hits)
+        if (hits > 0)
         {
             printf("Test failed.\n");
             exit(2);
         }
         /*endif*/
+        printf("Test passed.\n");
     }
     /*endif*/
 
+    if (decode_test_file)
+    {
+        printf("Decode file '%s'\n", decode_test_file);
+        modem_connect_tones_rx_init(&cng_rx, MODEM_CONNECT_TONES_FAX_CNG, cng_detected, NULL);
+        modem_connect_tones_rx_init(&ced_rx, MODEM_CONNECT_TONES_FAX_CED, ced_detected, NULL);
+        modem_connect_tones_rx_init(&ans_pr_rx, MODEM_CONNECT_TONES_ANS_PR, ec_dis_detected, NULL);
+        hits = 0;
+        if ((inhandle = afOpenFile(decode_test_file, "r", 0)) == AF_NULL_FILEHANDLE)
+        {
+            fprintf(stderr, "    Cannot open speech file '%s'\n", decode_test_file);
+            exit (2);
+        }
+        /*endif*/
+        if ((x = afGetFrameSize(inhandle, AF_DEFAULT_TRACK, 1)) != 2.0)
+        {
+            fprintf(stderr, "    Unexpected frame size in speech file '%s'\n", decode_test_file);
+            exit (2);
+        }
+        /*endif*/
+        if ((x = afGetRate(inhandle, AF_DEFAULT_TRACK)) != (float) SAMPLE_RATE)
+        {
+            fprintf(stderr, "    Unexpected sample rate in speech file '%s'\n", decode_test_file);
+            exit(2);
+        }
+        /*endif*/
+        if ((x = afGetChannels(inhandle, AF_DEFAULT_TRACK)) != 1.0)
+        {
+            fprintf(stderr, "    Unexpected number of channels in speech file '%s'\n", decode_test_file);
+            exit(2);
+        }
+        /*endif*/
+
+        when = 0;
+        hits = 0;
+        while ((frames = afReadFrames(inhandle, AF_DEFAULT_TRACK, amp, 8000)))
+        {
+            when++;
+            modem_connect_tones_rx(&cng_rx, amp, frames);
+            modem_connect_tones_rx(&ced_rx, amp, frames);
+            modem_connect_tones_rx(&ans_pr_rx, amp, frames);
+        }
+        /*endwhile*/
+        if (afCloseFile(inhandle) != 0)
+        {
+            fprintf(stderr, "    Cannot close speech file '%s'\n", decode_test_file);
+            exit(2);
+        }
+        /*endif*/
+        printf("    File gave %d hits.\n", hits);
+    }
     printf("Tests passed.\n");
     return  0;
 }
