@@ -22,14 +22,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: noise_tests.c,v 1.10 2007/11/10 11:14:58 steveu Exp $
+ * $Id: noise_tests.c,v 1.14 2008/05/13 13:17:26 steveu Exp $
  */
 
 /*! \page noise_tests_page Noise generator tests
 \section noise_tests_page_sec_1 What does it do?
 */
 
-#ifdef HAVE_CONFIG_H
+#if defined(HAVE_CONFIG_H)
 #include "config.h"
 #endif
 
