@@ -1,24 +1,26 @@
 #!/bin/sh
 #
-# spandsp regression tests
+# SpanDSP - a series of DSP components for telephony
+#
+# regression_tests.sh
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2, as
-# published by the Free Software Foundation.
+# it under the terms of the GNU Lesser General Public License version 2.1,
+# as published by the Free Software Foundation.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
+# You should have received a copy of the GNU Lesser General Public
+# License along with this program; if not, write to the Free Software
 # Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #
-# $Id: regression_tests.sh,v 1.47 2007/12/22 12:37:22 steveu Exp $
+# $Id: regression_tests.sh,v 1.51 2008/05/03 07:37:06 steveu Exp $
 #
 
-ITUTESTS_TIF=../itutests/fax/itutests.tif
+ITUTESTS_TIF=../test-data/itu/fax/itutests.tif
 STDOUT_DEST=xyzzy
 STDERR_DEST=xyzzy2
 
@@ -607,14 +609,15 @@ echo t4_tests completed OK
 #echo time_scale_tests completed OK
 echo time_scale_tests not enabled
 
-./tone_detect_tests >$STDOUT_DEST 2>$STDERR_DEST
-RETVAL=$?
-if [ $RETVAL != 0 ]
-then
-    echo tone_detect_tests failed!
-    exit $RETVAL
-fi
-echo tone_detect_tests completed OK
+#./tone_detect_tests >$STDOUT_DEST 2>$STDERR_DEST
+#RETVAL=$?
+#if [ $RETVAL != 0 ]
+#then
+#    echo tone_detect_tests failed!
+#    exit $RETVAL
+#fi
+#echo tone_detect_tests completed OK
+echo tone_detect_tests not enabled
 
 #./tone_generate_tests >$STDOUT_DEST 2>$STDERR_DEST
 #RETVAL=$?
