@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: super_tone_tx_tests.c,v 1.13 2006/10/24 13:45:29 steveu Exp $
+ * $Id: super_tone_tx_tests.c,v 1.22 2008/05/13 13:17:26 steveu Exp $
  */
 
 /*! \file */
@@ -31,27 +31,30 @@
 \section super_tone_tx_tests_page_sec_1 What does it do?
 */
 
-#ifdef HAVE_CONFIG_H
+#if defined(HAVE_CONFIG_H)
 #include "config.h"
 #endif
 
 #include <stdlib.h>
-#include <string.h>
-#include <strings.h>
 #include <stdio.h>
 #include <fcntl.h>
+#include <string.h>
+#include <strings.h>
 #include <ctype.h>
 #include <time.h>
 #include <inttypes.h>
 #include <sys/socket.h>
-#include <math.h>
-
-#include <libxml/xmlmemory.h>
-#include <libxml/parser.h>
-#include <libxml/xinclude.h>
-
 #include <audiofile.h>
-#include <tiffio.h>
+
+#if defined(HAVE_LIBXML_XMLMEMORY_H)
+#include <libxml/xmlmemory.h>
+#endif
+#if defined(HAVE_LIBXML_PARSER_H)
+#include <libxml/parser.h>
+#endif
+#if defined(HAVE_LIBXML_XINCLUDE_H)
+#include <libxml/xinclude.h>
+#endif
 
 #include "spandsp.h"
 
@@ -91,6 +94,7 @@ static void play_tones(super_tone_tx_state_t *tone, int max_samples)
 }
 /*- End of function --------------------------------------------------------*/
 
+#if defined(HAVE_LIBXML2)
 static int parse_tone(super_tone_tx_step_t **tree, xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur)
 {
     xmlChar *x;
@@ -192,7 +196,7 @@ static void parse_tone_set(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur)
             tone_tree = NULL;
             parse_tone(&tone_tree, doc, ns, cur);
             super_tone_tx_init(&tone, tone_tree);
-printf("Len %p %p %d %d\n", tone.levels[0], tone_tree, tone_tree->length, tone_tree->tone);
+//printf("Len %p %p %d %d\n", (void *) tone.levels[0], (void *) tone_tree, tone_tree->length, tone_tree->tone);
             play_tones(&tone, 99999999);
             super_tone_tx_free(tone_tree);
         }
@@ -274,28 +278,28 @@ static void get_tone_set(const char *tone_file, const char *set_id)
     xmlFreeDoc(doc);
 }
 /*- End of function --------------------------------------------------------*/
+#endif
 
 int main(int argc, char *argv[])
 {
-    filesetup = afNewFileSetup ();
-    if (filesetup == AF_NULL_FILESETUP)
+    if ((filesetup = afNewFileSetup ()) == AF_NULL_FILESETUP)
     {
     	fprintf(stderr, "    Failed to create file setup\n");
         exit(2);
     }
     afInitSampleFormat(filesetup, AF_DEFAULT_TRACK, AF_SAMPFMT_TWOSCOMP, 16);
     afInitRate(filesetup, AF_DEFAULT_TRACK, 8000.0);
-    //afInitCompression(filesetup, AF_DEFAULT_TRACK, AF_COMPRESSION_G711_ALAW);
     afInitFileFormat(filesetup, AF_FILE_WAVE);
     afInitChannels(filesetup, AF_DEFAULT_TRACK, 1);
 
-    outhandle = afOpenFile(OUT_FILE_NAME, "w", filesetup);
-    if (outhandle == AF_NULL_FILEHANDLE)
+    if ((outhandle = afOpenFile(OUT_FILE_NAME, "w", filesetup)) == AF_NULL_FILEHANDLE)
     {
         fprintf(stderr, "    Cannot open audio file '%s'\n", OUT_FILE_NAME);
         exit(2);
     }
+#if defined(HAVE_LIBXML2)
     get_tone_set("../spandsp/global-tones.xml", (argc > 1)  ?  argv[1]  :  "hk");
+#endif
     if (afCloseFile (outhandle) != 0)
     {
         fprintf(stderr, "    Cannot close audio file '%s'\n", OUT_FILE_NAME);
