@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: v17_tests.c,v 1.77 2007/12/29 04:16:29 steveu Exp $
+ * $Id: v17_tests.c,v 1.84 2008/07/16 17:01:49 steveu Exp $
  */
 
 /*! \page v17_tests_page V.17 modem tests
@@ -46,7 +46,7 @@ display of modem status is maintained.
 \section v17_tests_page_sec_2 How is it used?
 */
 
-#ifdef HAVE_CONFIG_H
+#if defined(HAVE_CONFIG_H)
 #include "config.h"
 #endif
 
@@ -134,42 +134,55 @@ static void reporter(void *user_data, int reason, bert_results_t *results)
 }
 /*- End of function --------------------------------------------------------*/
 
-static void v17putbit(void *user_data, int bit)
+static int v17_rx_status(void *user_data, int status)
 {
     v17_rx_state_t *rx;
     int i;
     int len;
     complexf_t *coeffs;
     
+    printf("V.17 rx status is %d\n", status);
     rx = (v17_rx_state_t *) user_data;
+    switch (status)
+    {
+    case PUTBIT_TRAINING_FAILED:
+        printf("Training failed\n");
+        break;
+    case PUTBIT_TRAINING_IN_PROGRESS:
+        printf("Training in progress\n");
+        break;
+    case PUTBIT_TRAINING_SUCCEEDED:
+        printf("Training succeeded\n");
+        len = v17_rx_equalizer_state(rx, &coeffs);
+        printf("Equalizer:\n");
+        for (i = 0;  i < len;  i++)
+            printf("%3d (%15.5f, %15.5f) -> %15.5f\n", i, coeffs[i].re, coeffs[i].im, powerf(&coeffs[i]));
+        break;
+    case PUTBIT_CARRIER_UP:
+        printf("Carrier up\n");
+        break;
+    case PUTBIT_CARRIER_DOWN:
+        printf("Carrier down\n");
+        break;
+    default:
+        printf("Eh! - %d\n", status);
+        break;
+    }
+    return 0;
+}
+/*- End of function --------------------------------------------------------*/
+
+static void v17putbit(void *user_data, int bit)
+{
+    v17_rx_state_t *rx;
+
     if (bit < 0)
     {
-        /* Special conditions */
-        switch (bit)
-        {
-        case PUTBIT_TRAINING_FAILED:
-            printf("Training failed\n");
-            break;
-        case PUTBIT_TRAINING_SUCCEEDED:
-            printf("Training succeeded\n");
-            len = v17_rx_equalizer_state(rx, &coeffs);
-            printf("Equalizer:\n");
-            for (i = 0;  i < len;  i++)
-                printf("%3d (%15.5f, %15.5f) -> %15.5f\n", i, coeffs[i].re, coeffs[i].im, powerf(&coeffs[i]));
-            break;
-        case PUTBIT_CARRIER_UP:
-            printf("Carrier up\n");
-            break;
-        case PUTBIT_CARRIER_DOWN:
-            printf("Carrier down\n");
-            break;
-        default:
-            printf("Eh!\n");
-            break;
-        }
+        v17_rx_status(user_data, bit);
         return;
     }
 
+    rx = (v17_rx_state_t *) user_data;
     if (decode_test_file)
         printf("Rx bit %d - %d\n", rx_bits++, bit);
     else
@@ -177,6 +190,24 @@ static void v17putbit(void *user_data, int bit)
 }
 /*- End of function --------------------------------------------------------*/
 
+static int v17_tx_status(void *user_data, int status)
+{
+    switch (status)
+    {
+    case MODEM_TX_STATUS_DATA_EXHAUSTED:
+        printf("V.17 tx data exhausted\n");
+        break;
+    case MODEM_TX_STATUS_SHUTDOWN_COMPLETE:
+        printf("V.17 tx shutdown complete\n");
+        break;
+    default:
+        printf("V.17 tx status is %d\n", status);
+        break;
+    }
+    return 0;
+}
+/*- End of function --------------------------------------------------------*/
+
 static int v17getbit(void *user_data)
 {
     return bert_get_bit(&bert);
@@ -386,6 +417,7 @@ int main(int argc, char *argv[])
         /* We will generate V.17 audio, and add some noise to it. */
         v17_tx_init(&tx, test_bps, tep, v17getbit, NULL);
         v17_tx_power(&tx, signal_level);
+        v17_tx_set_modem_status_handler(&tx, v17_tx_status, (void *) &tx);
         /* Move the carrier off a bit */
         tx.carrier_phase_rate = dds_phase_ratef(1792.0f);
         tx.carrier_phase = 0x40000000;
@@ -405,6 +437,7 @@ int main(int argc, char *argv[])
     }
 
     v17_rx_init(&rx, test_bps, v17putbit, &rx);
+    v17_rx_set_modem_status_handler(&rx, v17_rx_status, (void *) &rx);
     v17_rx_set_qam_report_handler(&rx, qam_report, (void *) &rx);
     span_log_set_level(&rx.logging, SPAN_LOG_SHOW_SEVERITY | SPAN_LOG_SHOW_PROTOCOL | SPAN_LOG_FLOW);
     span_log_set_tag(&rx.logging, "V.17 rx");
