@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: v27ter_tests.c,v 1.81 2007/12/29 04:16:29 steveu Exp $
+ * $Id: v27ter_tests.c,v 1.88 2008/07/16 17:01:49 steveu Exp $
  */
 
 /*! \page v27ter_tests_page V.27ter modem tests
@@ -45,7 +45,7 @@ display of modem status is maintained.
 \section v27ter_tests_page_sec_2 How is it used?
 */
 
-#ifdef HAVE_CONFIG_H
+#if defined(HAVE_CONFIG_H)
 #include "config.h"
 #endif
 
@@ -130,29 +130,39 @@ static void reporter(void *user_data, int reason, bert_results_t *results)
 }
 /*- End of function --------------------------------------------------------*/
 
+static int v27ter_rx_status(void *user_data, int status)
+{
+    printf("V.27ter rx status is %d\n", status);
+    switch (status)
+    {
+    case PUTBIT_TRAINING_FAILED:
+        printf("Training failed\n");
+        break;
+    case PUTBIT_TRAINING_IN_PROGRESS:
+        printf("Training in progress\n");
+        break;
+    case PUTBIT_TRAINING_SUCCEEDED:
+        printf("Training succeeded\n");
+        break;
+    case PUTBIT_CARRIER_UP:
+        printf("Carrier up\n");
+        break;
+    case PUTBIT_CARRIER_DOWN:
+        printf("Carrier down\n");
+        break;
+    default:
+        printf("Eh! - %d\n", status);
+        break;
+    }
+    return 0;
+}
+/*- End of function --------------------------------------------------------*/
+
 static void v27terputbit(void *user_data, int bit)
 {
     if (bit < 0)
     {
-        /* Special conditions */
-        switch (bit)
-        {
-        case PUTBIT_TRAINING_FAILED:
-            printf("Training failed\n");
-            break;
-        case PUTBIT_TRAINING_SUCCEEDED:
-            printf("Training succeeded\n");
-            break;
-        case PUTBIT_CARRIER_UP:
-            printf("Carrier up\n");
-            break;
-        case PUTBIT_CARRIER_DOWN:
-            printf("Carrier down\n");
-            break;
-        default:
-            printf("Eh!\n");
-            break;
-        }
+        v27ter_rx_status(user_data, bit);
         return;
     }
     if (decode_test_file)
@@ -162,6 +172,24 @@ static void v27terputbit(void *user_data, int bit)
 }
 /*- End of function --------------------------------------------------------*/
 
+static int v27ter_tx_status(void *user_data, int status)
+{
+    switch (status)
+    {
+    case MODEM_TX_STATUS_DATA_EXHAUSTED:
+        printf("V.27ter tx data exhausted\n");
+        break;
+    case MODEM_TX_STATUS_SHUTDOWN_COMPLETE:
+        printf("V.27ter tx shutdown complete\n");
+        break;
+    default:
+        printf("V.27ter tx status is %d\n", status);
+        break;
+    }
+    return 0;
+}
+/*- End of function --------------------------------------------------------*/
+
 static int v27tergetbit(void *user_data)
 {
     return bert_get_bit(&bert);
@@ -389,6 +417,7 @@ int main(int argc, char *argv[])
         /* We will generate V.27ter audio, and add some noise to it. */
         v27ter_tx_init(&tx, test_bps, tep, v27tergetbit, NULL);
         v27ter_tx_power(&tx, signal_level);
+        v27ter_tx_set_modem_status_handler(&tx, v27ter_tx_status, (void *) &tx);
         /* Move the carrier off a bit */
         tx.carrier_phase_rate = dds_phase_ratef(1810.0f);
 
@@ -403,6 +432,7 @@ int main(int argc, char *argv[])
     }
 
     v27ter_rx_init(&rx, test_bps, v27terputbit, NULL);
+    v27ter_rx_set_modem_status_handler(&rx, v27ter_rx_status, (void *) &rx);
     v27ter_rx_set_qam_report_handler(&rx, qam_report, (void *) &rx);
     span_log_set_level(&rx.logging, SPAN_LOG_SHOW_SEVERITY | SPAN_LOG_SHOW_PROTOCOL | SPAN_LOG_FLOW);
     span_log_set_tag(&rx.logging, "V.27ter-rx");
