@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: v42_tests.c,v 1.21 2007/11/10 11:14:59 steveu Exp $
+ * $Id: v42_tests.c,v 1.25 2008/05/13 13:17:26 steveu Exp $
  */
 
 /* THIS IS A WORK IN PROGRESS. IT IS NOT FINISHED. */
@@ -33,7 +33,7 @@ These tests connect two instances of V.42 back to back. V.42 frames are
 then exchanged between them.
 */
 
-#ifdef HAVE_CONFIG_H
+#if defined(HAVE_CONFIG_H)
 #include <config.h>
 #endif
 
